@@ -36,7 +36,6 @@ from metarace import timy
 from metarace import scbwin
 from metarace import tod
 from metarace import uiutil
-from metarace import eventdb
 from metarace import riderdb
 from metarace import strops
 from metarace import printing
@@ -67,17 +66,17 @@ RES_COL_STPTS = 11
 
 # scb consts
 SCB_INTSPRINT_NW = metarace.SCB_LINELEN - 8
-SCB_INTSPRINT_FMT = [(2, 'l'), (3, 'r'), ' ',
-                  (SCB_INTSPRINT_NW, 'l'), ' ', (1, 'r')]
+SCB_INTSPRINT_FMT = [(2, u'l'), (3, u'r'), u' ',
+                  (SCB_INTSPRINT_NW, u'l'), u' ', (1, u'r')]
 
 SCB_STARTERS_NW = metarace.SCB_LINELEN - 9
-SCB_STARTERS_FMT = [(3, 'r'), ' ', (SCB_STARTERS_NW,'l'), ' ', (4,'r')]
+SCB_STARTERS_FMT = [(3, u'r'), u' ', (SCB_STARTERS_NW,u'l'), u' ', (4,u'r')]
 SCB_RESMADISON_NW = metarace.SCB_LINELEN - 8
-SCB_RESMADISON_FMT = [(2,'r'),' ',(SCB_RESMADISON_NW,'l'),
-                      (2,'r'),(3,'r')]
+SCB_RESMADISON_FMT = [(2,u'r'),u' ',(SCB_RESMADISON_NW,u'l'),
+                      (2,u'r'),(3,u'r')]
 SCB_RESPOINTS_NW = metarace.SCB_LINELEN - 9
-SCB_RESPOINTS_FMT = [(2,'l'),(3,'r'),' ',
-                     (SCB_RESPOINTS_NW,'l'),(3,'r')]	# no club
+SCB_RESPOINTS_FMT = [(2,u'l'),(3,u'r'),u' ',
+                     (SCB_RESPOINTS_NW,u'l'),(3,u'r')]	# no club
 SPRINT_PLACE_DELAY = 3		# 3 seconds per place
 SPRINT_PLACE_DELAY_MAX = 11	# to a maximum of 11
 
@@ -317,17 +316,25 @@ class ps(object):
         ret = []
         sec = printing.section()
         sec.heading = 'Event ' + self.evno + ': ' + ' '.join([
-                       self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                       self.meet.edb.getvalue(self.event, eventdb.COL_INFO)
+                       self.event[u'pref'],
+                         self.event[u'info']
                               ]).strip()
+        lapstring = strops.lapstring(self.event[u'laps'])
+        substr = u' '.join([lapstring, self.event[u'dist'],
+                             self.event[u'prog']]).strip()
+        if substr:
+            sec.subheading = substr
         sec.units = 'Pts'
         fs = ''
         if self.finish is not None:
             fs = self.time_lbl.get_text().strip()
+        fl = None
+        ll = None
         for r in self.riders:
             rno = r[RES_COL_BIB]
-            rname = strops.resname(r[RES_COL_FIRST], r[RES_COL_LAST],
-                                    r[RES_COL_CLUB])
+            rname = strops.resname(r[RES_COL_FIRST].decode('utf-8'), 
+                                   r[RES_COL_LAST].decode('utf-8'),
+                                    r[RES_COL_CLUB].decode('utf-8'))
             infstr = ''
             plstr = ''
             if self.onestart and r[RES_COL_PLACE] is not None:
@@ -340,12 +347,27 @@ class ps(object):
                 finplace = ''
                 if r[RES_COL_FINAL] >= 0:
                     finplace = str(r[RES_COL_FINAL] + 1)
+
+                if self.scoring == 'madison':
+                    laps = r[RES_COL_LAPS]
+                    if fl is None:
+                        fl = laps # anddetermine laps down
+                    if ll is not None:
+                        down = fl - laps
+                        if ll != laps:
+                            sec.lines.append([None,None,unicode(down)
+                                              + u' Lap' + strops.plural(down)
+                                              + u' Behind', None, None, None])
+                    ll = laps
                 if plstr or finplace or ptstr: # only output those with points
                                          # dnf  or 
 					# placed in final sprint
                     sec.lines.append([plstr, rno, rname, None, fs, ptstr])
                 fs = ''
         ret.append(sec)
+
+        # output intermediate sprints?
+
         return ret
 
     def result_export(self, f):
@@ -357,8 +379,8 @@ class ps(object):
             df = str(dist) + 'm'
         hdr = ['Event ' + self.evno, '',
                ' '.join([
-                  self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                  self.meet.edb.getvalue(self.event, eventdb.COL_INFO)
+                  self.event[u'pref'],
+                         self.event[u'info']
                         ]).strip(),
                '', 'Pts', df]
         nopts = []
@@ -374,8 +396,9 @@ class ps(object):
             lapup = None
             for r in self.riders:
                 rno = r[RES_COL_BIB]
-                rname = strops.resname(r[RES_COL_FIRST], r[RES_COL_LAST],
-                                        r[RES_COL_CLUB])
+                rname =  strops.resname(r[RES_COL_FIRST].decode('utf-8'),
+                                   r[RES_COL_LAST].decode('utf-8'),
+                                    r[RES_COL_CLUB].decode('utf-8'))
                 infstr = ''
                 if r[RES_COL_INFO] is not None and r[RES_COL_INFO] != '':
                     infstr = r[RES_COL_INFO]
@@ -503,33 +526,54 @@ class ps(object):
 
     def startlist_report(self):
         """Return a startlist report."""
+        ret = []
         sec = printing.section()
-        sec.heading = ' '.join([
-                      'Event', self.evno, ':',
-                  self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                  self.meet.edb.getvalue(self.event, eventdb.COL_INFO),
-                      '- Start List'])
+        sec.heading = u' '.join([
+                      u'Event', self.evno, ':',
+                        self.event[u'pref'],
+                         self.event[u'info'],
+                      u'- Start List'])
+        lapstring = strops.lapstring(self.event[u'laps'])
+        substr = u' '.join([lapstring, self.event[u'dist'],
+                             self.event[u'prog']]).strip()
+        if substr:
+            sec.subheading = substr
         self.reorder_riderno()
         sec.lines = []
         cnt = 0
         for r in self.riders:
-            rno = r[RES_COL_BIB]
-            inf = r[RES_COL_INFO]
-            rname = strops.resname(r[RES_COL_FIRST], r[RES_COL_LAST],
-                                       r[RES_COL_CLUB])
+            rno = r[RES_COL_BIB].decode('utf-8')
+            inf = r[RES_COL_INFO].decode('utf-8')
+            first = r[RES_COL_FIRST].decode('utf-8')
+            last = r[RES_COL_LAST].decode('utf-8')
+            club = r[RES_COL_CLUB].decode('utf-8')
+            rname = strops.resname(first, last, club)
             sec.lines.append([None, rno, rname, inf, None, None])
             cnt += 1
         if cnt > 0:
             sec.lines.append([None, None, None])
             sec.lines.append([None, None, 'Total riders: ' + str(cnt)])
-        return sec
+        ret.append(sec)
+        return ret
 
     def do_startlist(self):
         """Show startlist on scoreboard."""
+        self.reorder_riderno()
         self.meet.scbwin = None
         self.timerwin = False
         startlist = []
+        self.meet.announce.gfx_overlay(1)
+        self.meet.announce.gfx_set_title(u'Startlist: '
+                            + self.event[u'pref'] + u' '
+                            + self.event[u'info'])
+
         for r in self.riders:
+            self.meet.announce.gfx_add_row([r[0],
+                               strops.resname(r[RES_COL_FIRST].decode('utf-8'),
+                                           r[RES_COL_LAST].decode('utf-8'),
+                                           r[RES_COL_CLUB].decode('utf-8')),
+                                                ''])
+
             if r[RES_COL_INRACE]:
                 startlist.append([r[RES_COL_BIB],
                                   strops.fitname(r[RES_COL_FIRST],
@@ -576,13 +620,14 @@ class ps(object):
             self.meet.announce.clrall()
             self.meet.ann_title(' '.join([
                   self.meet.event_string(self.evno), ':',
-                  self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                  self.meet.edb.getvalue(self.event, eventdb.COL_INFO)]))
+                     self.event[u'pref'],
+                         self.event[u'info']]))
 
             self.meet.announce.linefill(1, '_')
             self.meet.announce.linefill(8, '_')
 
             # fill in a sprint if not empty
+            sid = None
             i = self.ctrl_place_combo.get_active_iter()
             if i is not None:
                 pl = self.sprints.get_value(i, SPRINT_COL_PLACES)
@@ -603,6 +648,8 @@ class ps(object):
                         cnt += 1
                         if cnt > 3:	# is this required?
                             break
+                else:
+                    sid = int(self.sprints.get_string_from_iter(i))-1
 
             tp = ''
             if self.start is not None and self.finish is not None:
@@ -615,18 +662,25 @@ class ps(object):
 
             # do result standing
             mscount = len(self.sprints)
+            if sid is not None:
+                mscount = sid
+            sidstart = mscount - 8
+            if sidstart < 0:
+                sidstart = 0
+            elif sidstart > len(self.sprints)-10:
+                sidstart = len(self.sprints)-10
             if self.scoring == 'madison':
                 rtype = 'Team '
                 if self.evtype != 'madison':
                     rtype = 'Rider'
                 hdr = '     # ' + rtype + '                 Lap Pt '
                 nopts = ''
-                scnt = mscount
+                scnt = 0
                 for s in self.sprints:
-                    scnt -= 1
-                    if scnt < 10:
+                    if scnt >= sidstart and scnt < sidstart+10:
                         hdr += strops.truncpad(s[SPRINT_COL_ID], 4, 'r')
                         nopts += '    '
+                    scnt += 1
                 hdr += ' Fin'
                 self.meet.announce.setline(10, hdr)
                 curline = 11
@@ -652,11 +706,11 @@ class ps(object):
 
                     spstr = ''
                     if r[RES_COL_BIB] in self.auxmap:
-                        scnt = mscount
+                        scnt = 0
                         for s in self.auxmap[r[RES_COL_BIB]]:
-                            scnt -= 1
-                            if scnt < 10:
+                            if scnt >= sidstart and scnt < sidstart+10:
                                 spstr += str(s).rjust(4)
+                            scnt += 1
                     else:
                         spstr = nopts
 
@@ -721,6 +775,22 @@ class ps(object):
 
     def do_places(self):
         """Show race result on scoreboard."""
+
+        thesec = self.result_report()
+        placestype = u'Leaders: '
+        if len(thesec) > 0:
+            if self.finished: 
+                placestype = u'Result: '
+            self.meet.announce.gfx_overlay(1)
+            self.meet.announce.gfx_set_title(placestype
+                                + self.event[u'pref'] + u' '
+                                + self.event[u'info'])
+            for l in thesec[0].lines:
+                pts = '-'
+                if l[5]:
+                    pts = l[5]
+                self.meet.announce.gfx_add_row([l[0], l[2], pts])
+
         resvec = []
         fmt = ''
         hdr = ''
@@ -756,7 +826,7 @@ class ps(object):
         self.meet.scbwin = None
         self.timerwin = False
         self.meet.scbwin = scbwin.scbtable(self.meet.scb,
-                                           hdr, fmt, resvec, delay=50, pagesz=7)
+                                           hdr, fmt, resvec, delay=60, pagesz=7)
         self.meet.scbwin.reset()
         return False
 
@@ -813,8 +883,8 @@ class ps(object):
             # and do it on the scoreboard too ?!
             self.meet.scbwin = scbwin.scbintsprint(self.meet.scb,
                                    self.meet.racenamecat(self.event),
-                                   'Gaining a Lap',
-                                   [(3,'r'), ' ', (metarace.SCB_LINELEN-4,'l')],
+                                   u'Gaining a Lap',
+                                   [(3,u'r'), u' ', (metarace.SCB_LINELEN-4,u'l')],
                                    srlines)
             self.meet.scbwin.reset()
             self.next_sprint_counter += 1
@@ -835,7 +905,7 @@ class ps(object):
                 r[RES_COL_LAPS] -= 1
                 recalc = True
                 self.log.info('Rider ' + str(bib) + ' lose lap')
-                rlines.append(' '.join([bib.rjust(3), 
+                rlines.append(u' '.join([bib.rjust(3), 
                                   strops.fitname(r[RES_COL_FIRST],
                                        r[RES_COL_LAST], 26, trunc=True)]))
             else:
@@ -859,8 +929,8 @@ class ps(object):
         """Show race timer on scoreboard."""
         tp = 'Time:'
         self.meet.scbwin = scbwin.scbtimer(self.meet.scb,
-                    self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                    self.meet.edb.getvalue(self.event, eventdb.COL_INFO),
+                                       self.event[u'pref'],
+                         self.event[u'info'],
                                tp)
         self.timerwin = True
         self.meet.scbwin.reset()
@@ -1097,16 +1167,6 @@ class ps(object):
         self.info_expand.set_label('Race Info : '
                     + self.meet.racenamecat(self.event, 64))
 
-    def ps_info_prefix_changed_cb(self, entry, data=None):
-        """Update event prefix."""
-        self.meet.edb.editevent(self.event, prefix=entry.get_text())
-        self.update_expander_lbl_cb()
-
-    def ps_info_title_changed_cb(self, entry, data=None):
-        """Update event title."""
-        self.meet.edb.editevent(self.event, info=entry.get_text())
-        self.update_expander_lbl_cb()
-
     def ps_info_time_edit_clicked_cb(self, button, data=None):
         """Run the edit times dialog."""
         ostx = ''
@@ -1217,6 +1277,14 @@ class ps(object):
                 if delay > SPRINT_PLACE_DELAY_MAX:
                     delay = SPRINT_PLACE_DELAY_MAX
                 self.oktochangecombo = True
+
+                self.meet.announce.gfx_overlay(1)
+                self.meet.announce.gfx_set_title(self.event[u'pref'] + u' '
+                                             + self.event[u'info'] + u' '
+                                                  + sinfo)
+                for r in self.sprintresults[sid]:
+                    self.meet.announce.gfx_add_row([r[0], r[4], r[3]])
+
                 glib.timeout_add_seconds(delay, self.delayed_result)
             elif type(self.meet.scbwin) is scbwin.scbtable:
                 self.do_places() # overwrite result table?
@@ -1331,11 +1399,15 @@ class ps(object):
                         self.auxmap[bib][index] = str(points[place])
                     if final:
                         r[RES_COL_FINAL] = place
-                    self.sprintresults[index].append([str(place + 1),
+                        self.finished = True
+                    plstr = unicode(place + 1) + u'.'
+                    fname = r[RES_COL_FIRST].decode('utf-8')
+                    lname = r[RES_COL_LAST].decode('utf-8')
+                    club = r[RES_COL_CLUB].decode('utf-8')
+                    self.sprintresults[index].append([plstr,
                            r[RES_COL_BIB],
-                           strops.fitname(r[RES_COL_FIRST],
-                                     r[RES_COL_LAST], SCB_INTSPRINT_NW),
-                                                         ptsstr])
+                           strops.fitname(fname, lname, SCB_INTSPRINT_NW),
+                                ptsstr, strops.resname(fname, lname, club)])
                     count += 1
                 else:
                     self.log.error('Ignoring duplicate no: ' +repr(bib))
@@ -1409,6 +1481,7 @@ class ps(object):
     # result recalculation
     def recalculate(self):
         self.zeropoints()
+        self.finished = False
         self.auxmap = {}
         idx = 0
         for s in self.sprints:
@@ -1517,13 +1590,21 @@ class ps(object):
         """Hide race window."""
         self.frame.hide()
 
+    def editent_cb(self, entry, col):
+        """Shared event entry update callback."""
+        if col == u'pref':
+            self.event[u'pref'] = entry.get_text().decode('utf-8', 'replace')
+        elif col == u'info':
+            self.event[u'info'] = entry.get_text().decode('utf-8', 'replace')
+        self.update_expander_lbl_cb()
+
     def __init__(self, meet, event, ui=True):
         """Constructor."""
         self.meet = meet
         self.event = event      # Note: now a treerowref
-        self.evno = meet.edb.getvalue(event, eventdb.COL_EVNO)
-        self.evtype = meet.edb.getvalue(event, eventdb.COL_TYPE)
-        self.series = meet.edb.getvalue(event, eventdb.COL_SERIES)
+        self.evno = event[u'evid']
+        self.evtype = event[u'type']
+        self.series = event[u'seri']
         self.configpath = meet.event_configfile(self.evno)
 
         self.log = logging.getLogger('points')
@@ -1557,6 +1638,7 @@ class ps(object):
         self.next_sprint_counter = 0
         self.oktochangecombo = False
         self.autospec = ''
+        self.finished = False
 
         # data models
         self.sprints = gtk.ListStore(gobject.TYPE_STRING,   # ID = 0
@@ -1590,11 +1672,11 @@ class ps(object):
         b.get_object('ps_info_evno').set_text(self.evno)
         self.showev = b.get_object('ps_info_evno_show')
         self.prefix_ent = b.get_object('ps_info_prefix')
-        self.prefix_ent.set_text(self.meet.edb.getvalue(
-                   self.event, eventdb.COL_PREFIX))
+        self.prefix_ent.connect('changed', self.editent_cb,u'pref')
+        self.prefix_ent.set_text(self.event[u'pref'])
         self.info_ent = b.get_object('ps_info_title')
-        self.info_ent.set_text(self.meet.edb.getvalue(
-                   self.event, eventdb.COL_INFO))
+        self.info_ent.connect('changed', self.editent_cb,u'info')
+        self.info_ent.set_text(self.event[u'info'])
 
         self.time_lbl = b.get_object('ps_info_time')
         self.time_lbl.modify_font(pango.FontDescription("monospace bold"))
