@@ -36,7 +36,6 @@ explicit series confirmation before import into meet succeeds.
 
 """
 
-import csv
 import gtk
 import glib
 import gobject
@@ -45,6 +44,7 @@ import os
 
 from metarace import uiutil
 from metarace import strops
+from metarace import ucsv
 
 # Model column constants
 COL_BIB = 0
@@ -57,13 +57,14 @@ COL_REFID = 6
 COL_UCICODE = 7
 
 # Reserved rider numbers.
-RESERVED_NOS = ['bib', 'no.', 'no', 'all', 'number', 'num', '#']
+RESERVED_NOS = [u'bib', u'no.', u'no', u'all', u'number', u'num', u'#']
 
 class riderdb(object):
     """Rider database."""
-    def addempty(self, bib='', series=''):
+    def addempty(self, bib=u'', series=u''):
         """Add a new empty row in the rider model."""
-        i = self.model.append([bib.lower(), '', '', '', '', series, '', ''])
+        i = self.model.append([bib.lower(), u'', u'', u'', u'',
+                                series, u'', u''])
         ref = gtk.TreeRowReference(self.model, self.model.get_path(i))
         if self.view is not None:
             self.postedit = None
@@ -79,15 +80,15 @@ class riderdb(object):
         """Load refids from the named chipfile into to the riderdb."""
         count = 0
         if os.path.isfile(csvfile):
-            self.log.debug('Loading refids from %s', csvfile)
+            self.log.debug(u'Loading refids from ' + repr(csvfile))
             with open(csvfile, 'rb') as f:
-                cr = csv.reader(f)
+                cr = ucsv.UnicodeReader(f)
                 for row in cr:
-                    ir = [cell.translate(strops.PRINT_TRANS) for cell in row]
-                    if len(ir) > 1 and ir[0] != 'Refid':
+                    ir = [cell.translate(strops.PRINT_UTRANS) for cell in row]
+                    if len(ir) > 1 and ir[0] != u'Refid':
                         nrefid = ir[0].strip().lower()
                         bib = ir[1].strip().lower()
-                        series = ''
+                        series = u''
                         if len(ir) > 2:
                             series = ir[2].strip()
 
@@ -101,40 +102,41 @@ class riderdb(object):
                                                refid=nrefid)
                             count += 1
                         else:
-                            self.log.warn('Invalid No.: %s', bib)
-            self.log.debug('Loaded ' + str(count) + ' refids from ' 
-                            + str(csvfile))
+                            self.log.warn(u'Invalid No.: %s', bib)
+            self.log.debug(u'Loaded ' + unicode(count) + u' refids from ' 
+                            + repr(csvfile))
 
     def load(self, csvfile=None):
         """Load riders from supplied CSV file."""
         count = 0
         if os.path.isfile(csvfile):
-            self.log.debug('Loading riders from %s', csvfile)
+            self.log.debug(u'Loading riders from ' + repr(csvfile))
             with open(csvfile, 'rb') as f:
-                cr = csv.reader(f)
+                cr = ucsv.UnicodeReader(f)
                 for row in cr:
-                    ir = [cell.translate(strops.PRINT_TRANS) for cell in row]
-                    if len(ir) > 0 and ir[0] != 'Bib' and ir[0] != 'No.':
+                    ir = [cell.translate(strops.PRINT_UTRANS) for cell in row]
+                    if len(ir) > 0 and ir[0] != u'Bib' and ir[0] != u'No.':
                         bib = ir[COL_BIB].strip().lower()
                         if bib.isalnum() and bib not in RESERVED_NOS:
-                            nr = [bib, '', '', '', '', '', '', '']
+                            nr = [bib, u'', u'', u'', u'', u'', u'', u'']
                             for i in range(1,8):
                                 if len(ir) > i:
                                     nr[i] = ir[i].strip()
                             # overwrite refid in load data if chipfile loaded
                             bibser = strops.bibser2bibstr(bib, nr[COL_SERIES])
                             if bibser in self.maptag:
-                                if nr[COL_REFID] == '':
+                                if nr[COL_REFID] == u'':
                                     nr[COL_REFID] = self.maptag[bibser]
                             if self.getrider(bib, nr[COL_SERIES]) is None:
                                 self.model.append(nr)
                                 count += 1
                             else:
-                                self.log.warn('Duplicate no. ignored: %s', bib)
+                                self.log.warn(u'Duplicate no. ignored: '
+                                              + repr(bib))
                         else:
-                            self.log.warn('Invalid No.: %s', bib)
-            self.log.debug('Loaded ' + str(count) + ' riders from ' 
-                            + str(csvfile))
+                            self.log.warn(u'Invalid No.: ' + repr(bib))
+            self.log.debug('Loaded ' + unicode(count) + ' riders from ' 
+                            + repr(csvfile))
 
     def save_chipfile(self, csvfile=None):
         """Save refids from current model to supplied CSV file."""
@@ -144,13 +146,13 @@ class riderdb(object):
         if os.path.exists(csvfile):
             # load in old chipfile 
             with open(csvfile, 'rb') as f:
-                cr = csv.reader(f)
+                cr = ucsv.UnicodeReader(f)
                 for row in cr:
-                    ir = [cell.translate(strops.PRINT_TRANS) for cell in row]
-                    if len(ir) > 1 and ir[0] != 'Refid':
+                    ir = [cell.translate(strops.PRINT_UTRANS) for cell in row]
+                    if len(ir) > 1 and ir[0] != u'Refid':
                         nrefid = ir[0].strip().lower()
                         bib = ir[1].strip().lower()
-                        series = ''
+                        series = u''
                         if len(ir) > 2:
                             series = ir[2].strip()
 
@@ -161,47 +163,51 @@ class riderdb(object):
                                 chiplist.append(bibstr)
                                 count += 1
                         else:
-                            self.log.warn('Invalid No.: %s', bib)
-                self.log.debug('Loaded ' + str(count) + ' refids from ' 
-                            + str(csvfile))
+                            self.log.warn(u'Invalid No.: ' + repr(bib))
+                self.log.debug(u'Loaded ' + unicode(count) + u' refids from ' 
+                            + repr(csvfile))
 
         # overwrite any existing refids
         ocount = 0
         ncount = 0
         for r in self.model:
-            if r[COL_REFID]:
-                bibstr = strops.bibser2bibstr(r[COL_BIB], r[COL_SERIES])
+            urefid = r[COL_REFID].decode('utf-8')
+            if urefid:
+                ubib = r[COL_BIB].decode('utf-8')
+                useries = r[COL_SERIES].decode('utf-8')
+                bibstr = strops.bibser2bibstr(ubib, useries)
                 if bibstr not in chipmap:
                     chiplist.append(bibstr)
                     ncount += 1
                 else:
-                    if r[COL_REFID] != chipmap[bibstr]:
+                    if urefid != chipmap[bibstr]:
                         ocount += 1
-                chipmap[bibstr] = r[COL_REFID]
-        self.log.debug('Replacing ' + str(ocount) + ' refids from model.')
-        self.log.debug('Adding ' + str(ncount) + ' refids from model.')
+                chipmap[bibstr] = urefid
+        self.log.debug(u'Replacing ' + unicode(ocount) + ' refids from model.')
+        self.log.debug(u'Adding ' + unicode(ncount) + ' refids from model.')
 
         chiplist.sort(key=strops.bibstr_key)
         count = 0
         with open(csvfile, 'wb') as f:
-            cr = csv.writer(f)
-            cr.writerow(["Refid","No.","Series"])
+            cr = ucsv.UnicodeWriter(f)
+            cr.writerow([u'Refid',u'No.',u'Series'])
             for bs in chiplist:
                 (bib, ser) = strops.bibstr2bibser(bs)
                 refid = chipmap[bs]
                 if refid:
                     count += 1
                     cr.writerow([refid, bib, ser])
-        self.log.debug('Wrote ' + str(count) + ' refids to: ' + repr(csvfile))
+        self.log.debug(u'Wrote ' + unicode(count) + u' refids to: '
+                           + repr(csvfile))
 
     def save(self, csvfile=None):
         """Save current model to supplied CSV file."""
-        self.log.debug('Saving riders to %s', csvfile)
+        self.log.debug(u'Saving riders to '+ repr(csvfile))
         with open(csvfile, 'wb') as f:
-            cr = csv.writer(f)
-            cr.writerow(["No.","First Name","Last Name","Club",
-                         "Category","Series(optional)","Refid","UCIcode"])
-            cr.writerows(self.model)
+            cr = ucsv.UnicodeWriter(f)
+            cr.writerow([u'No.',u'First Name',u'Last Name',u'Club',
+                         u'Category',u'Series(optional)',u'Refid',u'UCIcode'])
+            cr.writerows(self)	# check the unicode out-in-out-in
 
     def gotorow(self, ref=None):
         """Move view selection to the specified row reference."""
@@ -289,14 +295,14 @@ class riderdb(object):
         self.view = v
         return self.view
 
-    def getrider(self, bib, series=''):
+    def getrider(self, bib, series=u''):
         """Return a reference to the row with the given bib and series."""
         ret = None
         bib = bib.lower()
         i = self.model.get_iter_first()
         while i is not None:
-            if (self.model.get_value(i, COL_BIB) == bib
-                and self.model.get_value(i, COL_SERIES) == series):
+            if (self.get_value(i, COL_BIB) == bib
+                and self.get_value(i, COL_SERIES) == series):
                 ret = gtk.TreeRowReference(self.model,
                                            self.model.get_path(i))
                 break
@@ -307,9 +313,10 @@ class riderdb(object):
         """Try and return a new unique rider number string."""
         lmax = 1
         for r in self.model:
-            if r[COL_BIB].isdigit() and int(r[COL_BIB]) >= lmax:
-                lmax = int(r[COL_BIB]) + 1
-        return str(lmax)
+            ubib = r[COL_BIB].decode('utf-8')
+            if ubib.isdigit() and int(ubib) >= lmax:
+                lmax = int(ubib) + 1
+        return unicode(lmax)
 
     def getrefid(self, refid):
         """Return a reference to the row with the given refid."""
@@ -317,33 +324,33 @@ class riderdb(object):
         ck = refid.lower()
         i = self.model.get_iter_first()
         while i is not None:
-            if self.model.get_value(i, COL_REFID).lower() == ck:
+            if self.get_value(i,COL_REFID).lower()==ck:
                 ret = gtk.TreeRowReference(self.model,
                                            self.model.get_path(i))
                 break
             i = self.model.iter_next(i)
         return ret
         
-    def biblistfromcat(self, cat, series=''):
+    def biblistfromcat(self, cat, series=u''):
         """Return a list of rider numbers in the given cat."""
-        ret = ''
+        ret = u''
         cat = cat.upper()
         i = self.model.get_iter_first()
         while i is not None:
-            if (self.model.get_value(i, COL_CAT).upper() == cat
-               and self.model.get_value(i, COL_SERIES) == series):
-                ret += ' ' + self.model.get_value(i, COL_BIB)
+            if (self.get_value(i, COL_CAT).upper() == cat
+               and self.get_value(i, COL_SERIES) == series):
+                ret += u' ' + self.get_value(i, COL_BIB)
             i = self.model.iter_next(i)
         return ret
-        
-    def getbibs(self, cat, series=''):
+
+    def getbibs(self, cat, series=u''):
         """Return a list of refs to riders in the given cat and series."""
         ret = []
         cat = cat.upper()
         i = self.model.get_iter_first()
         while i is not None:
-            if (self.model.get_value(i, COL_CAT).upper() == cat
-               and self.model.get_value(i, COL_SERIES) == series):
+            if (self.get_value(i, COL_CAT).upper() == cat
+               and self.get_value(i, COL_SERIES) == series):
                 ret.append(gtk.TreeRowReference(self.model,
                                            self.model.get_path(i)))
             i = self.model.iter_next(i)
@@ -353,24 +360,32 @@ class riderdb(object):
         """Return a list of all the series in the rider db."""
         ret = []
         for row in self.model:
-            if row[COL_SERIES] not in ret:
-                ret.append(row[COL_SERIES])
+            useries = row[COL_SERIES].decode('utf-8')
+            if useries not in ret:
+                ret.append(useries)
         return ret
 
     def listcats(self, series=None):
         """Return a list of all the categories in the specified series."""
         ret = []
         for row in self.model:
-            if row[COL_CAT].upper() not in ret and (
-                    series is None or row[COL_SERIES] == series):
-                ret.append(row[COL_CAT].upper())
+            ucat = row[COL_CAT].decode('utf-8').upper()
+            useries = row[COL_SERIES].decode('utf-8')
+            if ucat not in ret and (series is None or useries == series):
+                ret.append(ucat)
         return ret
 
+    def get_value(self, iter, column):
+        return self.model.get_value(iter, column).decode('utf-8')
+
+    def get_rowvalue(self, row, column):
+        return row[column].decode('utf-8')
+
     def getvalue(self, ref, col):
         """Return the specified column from the supplied row."""
         ret = None
         if ref.valid():
-            ret = self.model[ref.get_path()][col]
+            ret = self.model[ref.get_path()][col].decode('utf-8')
         return ret
 
     def editrider(self, ref=None, first=None, last=None,
@@ -381,7 +396,7 @@ class riderdb(object):
             if bib is None:
                 bib = self.nextriderno()
             ## ERROR: num not ever defined?
-            i = self.model.append([bib, '', '', '', '', '', '', ''])
+            i = self.model.append([bib, u'', u'', u'', u'', u'', u'', u''])
             ref = gtk.TreeRowReference(self.model,
                                            self.model.get_path(i))
         if ref.valid():
@@ -411,11 +426,11 @@ class riderdb(object):
         ret = False
         if new_text is not None:
             new_text = new_text.strip()
-            if new_text != self.model[path][col]:
+            if new_text != self.model[path][col].decode('utf-8'):
                 if col == COL_BIB:
                     if new_text.isalnum() and new_text.lower != 'all':
                         if not self.getrider(new_text,
-                                             self.model[path][COL_SERIES]):
+                                 self.model[path][COL_SERIES].decode('utf-8')):
                             self.model[path][COL_BIB] = new_text.lower()
                             ret = True
                         else:
@@ -426,7 +441,8 @@ class riderdb(object):
                     else:
                         self.log.warn('Invalid no. number ignored.')
                 elif col == COL_SERIES:
-                    if not self.getrider(self.model[path][COL_BIB], new_text):
+                    if not self.getrider(
+                          self.model[path][COL_BIB].decode('utf-8'), new_text):
                         self.model[path][COL_SERIES] = new_text
                         ret = True
                     else:  # This path is almost never a real problem
@@ -555,8 +571,9 @@ class riderdb(object):
         return bool(self.model.get_value(iter, data))
 
     def __iter__(self):
-        """Return a generator for the rider model."""
-        return uiutil.liststore_inorder(self.model)
+        """Map out unicode strings."""
+        for r in self.model:
+            yield [a.decode('utf-8') for a in r]
 
     def __init__(self):
         """Constructor for the rider db.
