@@ -33,8 +33,6 @@ import gobject
 import pango
 import os
 import logging
-import csv
-import ConfigParser
 import threading
 
 import metarace
@@ -44,6 +42,7 @@ from metarace import riderdb
 from metarace import strops
 from metarace import uiutil
 from metarace import printing
+from metarace import jsonconfig
 
 # Model columns
 
@@ -67,7 +66,7 @@ COL_RFSEEN = 13		# list of tods this rider 'seen' by rfid
 
 # rider commands
 RIDER_COMMANDS_ORD = [ 'add', 'del', 'que', 'dns', 'hd',
-                   'dnf', 'dsq', 'com', 'nst', 'ret',
+                   'dnf', 'dsq', 'com', 'ret',
                    '', 'fin']	# then intermediates...
 RIDER_COMMANDS = {'dns':'Did not start',
                    'hd':'Outside time limit',
@@ -78,7 +77,6 @@ RIDER_COMMANDS = {'dns':'Did not start',
                    'que':'Query riders',
                    'fin':'Final places',
                    'com':'Add comment',
-                   'nst':'Add new start',
                    'ret':'Return to race',
                    '':'',
                    }
@@ -104,7 +102,7 @@ key_confirm = 'F8'
 key_undo = 'Z'
 
 # config version string
-EVENT_ID = 'roadrace-1.3'
+EVENT_ID = 'roadrace-2.0'
 
 def key_bib(x):
     """Sort on bib field of aux row."""
@@ -131,144 +129,145 @@ class rms(object):
         self.resettimer()
         self.cats = []
 
-        cr = ConfigParser.ConfigParser({'start':'',
-                                        'lstart':'',
-                                        'id':EVENT_ID,
-                                        'finish':'',
-                                        'finished':'No',
-                                        'places':'',
-                                        'comment':'',
-                                        'hidecols':'',
-                                        'categories':'',
-                                        'intermeds':'',
-                                        'contests':'',
-                                        'tallys':'',
-                                        'nolaps':'False',
-                                        'startlist':''})
-        cr.add_section('event')
-        cr.add_section('riders')
+        cr = jsonconfig.config({u'event':{u'start':u'',
+                                        u'lstart':u'',
+                                        u'id':EVENT_ID,
+                                        u'finish':u'',
+                                        u'finished':u'No',
+                                        u'places':u'',
+                                        u'comment':[],
+                                        u'hidecols':[],	# TODO: default hides
+                                        u'categories':u'',
+                                        u'intermeds':[],
+                                        u'contests':[],
+                                        u'tallys':[],
+                                        u'nolaps':u'False',
+                                        u'startlist':u'',
+                                        u'resultcats':u'False'}})
+        cr.add_section(u'event')
+        cr.add_section(u'riders')
         # sections for commissaire awarded bonus/penalty
-        cr.add_section('stagebonus')
-        cr.add_section('stagepenalty')
-        if os.path.isfile(self.configpath):
-            self.log.debug('Attempting to read config from path='
-                            + repr(self.configpath))
-            cr.read(self.configpath)
+        cr.add_section(u'stagebonus')
+        cr.add_section(u'stagepenalty')
+
+        # check for config file
+        try:
+            with open(self.configpath, 'rb') as f:
+                cr.read(f)
+        except Exception as e:
+            self.log.error(u'Reading event config: ' + unicode(e))
 
         # load _result_ categories
-        catlist = cr.get('event', 'categories')
-        if catlist == 'AUTO':
+        catlist = cr.get(u'event', u'categories')
+        if u'AUTO' in catlist:	# ignore any others and re-load from rdb
             self.cats = self.meet.rdb.listcats()
             self.autocats = True
         else:
             self.autocats = False
-            for cat in catlist.split(','):
-                if cat != '':
+            for cat in catlist:
+                if cat != u'':
                     cat = cat.upper()
                     self.cats.append(cat)
-        self.cats.append('')	# always include one empty cat
+        self.cats.append(u'')	# always include one empty cat
 
         # restore intermediates
-        for i in cr.get('event', 'intermeds').translate(
-                            strops.BIBLIST_TRANS).split():
+        for i in cr.get(u'event', u'intermeds'):
             if i in RESERVED_SOURCES:
-                self.log.info('Ignoring reserved intermediate: ' + repr(i))
+                self.log.info(u'Ignoring reserved intermediate: ' + repr(i))
             else:
-                crkey = 'intermed_' + i
-                descr = ''
-                places = ''
-                if cr.has_option(crkey, 'descr'):
-                    descr = cr.get(crkey, 'descr')
-                if cr.has_option(crkey, 'places'):
+                crkey = u'intermed_' + i
+                descr = u''
+                places = u''
+                if cr.has_option(crkey, u'descr'):
+                    descr = cr.get(crkey, u'descr')
+                if cr.has_option(crkey, u'places'):
                     places = strops.reformat_placelist(
-                                 cr.get(crkey, 'places'))
+                                 cr.get(crkey, u'places'))
                 if i not in self.intermeds:
-                    self.log.debug('Adding intermediate: '
-                                    + repr(i) + ':' + descr
-                                    + ':' + places)
+                    self.log.debug(u'Adding intermediate: '
+                                    + repr(i) + u':' + descr
+                                    + u':' + places)
                     self.intermeds.append(i)
-                    self.intermap[i] = {'descr':descr, 'places':places}
+                    self.intermap[i] = {u'descr':descr, u'places':places}
                 else:
-                    self.log.info('Ignoring duplicate intermediate: '
+                    self.log.info(u'Ignoring duplicate intermediate: '
                                    + repr(i))
 
         # load contest meta data
         tallyset = set()
-        for i in cr.get('event', 'contests').translate(
-                            strops.BIBLIST_TRANS).split():
+        for i in cr.get('event', 'contests'):
             if i not in self.contests:
                 self.contests.append(i)
                 self.contestmap[i] = {}
-                crkey = 'contest_' + i
-                tally = ''
-                if cr.has_option(crkey, 'tally'):
-                    tally = cr.get(crkey, 'tally')
+                crkey = u'contest_' + i
+                tally = u''
+                if cr.has_option(crkey, u'tally'):
+                    tally = cr.get(crkey, u'tally')
                     if tally:
                         tallyset.add(tally)
-                self.contestmap[i]['tally'] = tally
+                self.contestmap[i][u'tally'] = tally
                 descr = i
-                if cr.has_option(crkey, 'descr'):
-                    descr = cr.get(crkey, 'descr')
-                    if descr == '':
+                if cr.has_option(crkey, u'descr'):
+                    descr = cr.get(crkey, u'descr')
+                    if descr == u'':
                         descr = i
-                self.contestmap[i]['descr'] = descr
+                self.contestmap[i][u'descr'] = descr
                 labels = []
-                if cr.has_option(crkey, 'labels'):
-                    labels = cr.get(crkey, 'labels').split()
-                self.contestmap[i]['labels'] = labels
+                if cr.has_option(crkey, u'labels'):
+                    labels = cr.get(crkey, u'labels').split()
+                self.contestmap[i][u'labels'] = labels
                 source = i
-                if cr.has_option(crkey, 'source'):
-                    source = cr.get(crkey, 'source')
-                    if source == '':
+                if cr.has_option(crkey, u'source'):
+                    source = cr.get(crkey, u'source')
+                    if source == u'':
                         source = i
-                self.contestmap[i]['source'] = source
+                self.contestmap[i][u'source'] = source
                 bonuses = []
-                if cr.has_option(crkey, 'bonuses'):
-                    for bstr in cr.get(crkey, 'bonuses').split():
+                if cr.has_option(crkey, u'bonuses'):
+                    for bstr in cr.get(crkey, u'bonuses').split():
                         bt = tod.str2tod(bstr)
                         if bt is None:
-                            self.log.info('Invalid bonus ' + repr(bstr)
-                              + ' in contest ' + repr(i))
+                            self.log.info(u'Invalid bonus ' + repr(bstr)
+                              + u' in contest ' + repr(i))
                             bt = tod.ZERO
                         bonuses.append(bt)
-                self.contestmap[i]['bonuses'] = bonuses
+                self.contestmap[i][u'bonuses'] = bonuses
                 points = []
-                if cr.has_option(crkey, 'points'):
-                    pliststr = cr.get(crkey, 'points').strip()
-                    if pliststr and tally == '': # no tally for these points!
-                        self.log.error('No tally for points in contest: '
+                if cr.has_option(crkey, u'points'):
+                    pliststr = cr.get(crkey, u'points').strip()
+                    if pliststr and tally == u'': # no tally for these points!
+                        self.log.error(u'No tally for points in contest: '
                                         + repr(i))
-                        tallyset.add('')	# add empty placeholder
+                        tallyset.add(u'')	# add empty placeholder
                     for pstr in pliststr.split():
                         pt = 0
                         try:
                             pt = int(pstr)
                         except:
-                            self.log.info('Invalid points ' + repr(pstr)
-                              + ' in contest ' + repr(i))
+                            self.log.info(u'Invalid points ' + repr(pstr)
+                              + u' in contest ' + repr(i))
                         points.append(pt)
-                self.contestmap[i]['points'] = points
+                self.contestmap[i][u'points'] = points
                 allsrc = False		# all riders in source get same pts
-                if cr.has_option(crkey, 'all_source'):
-                    allsrc = strops.confopt_bool(cr.get(crkey, 'all_source'))
-                self.contestmap[i]['all_source'] = allsrc
+                if cr.has_option(crkey, u'all_source'):
+                    allsrc = strops.confopt_bool(cr.get(crkey, u'all_source'))
+                self.contestmap[i][u'all_source'] = allsrc
             else:
-                self.log.info('Ignoring duplicate contest: ' + repr(i))
+                self.log.info(u'Ignoring duplicate contest: ' + repr(i))
 
             # check for invalid allsrc
-            if self.contestmap[i]['all_source']:
-                if (len(self.contestmap[i]['points']) > 1 
-                    or len(self.contestmap[i]['bonuses']) > 1):
-                    self.log.info('Ignoring extra points/bonus for '
-                                  + 'all source contest ' + repr(i))
+            if self.contestmap[i][u'all_source']:
+                if (len(self.contestmap[i][u'points']) > 1 
+                    or len(self.contestmap[i][u'bonuses']) > 1):
+                    self.log.info(u'Ignoring extra points/bonus for '
+                                  + u'all source contest ' + repr(i))
      
         # load points tally meta data
-        tallylist = cr.get('event', 'tallys').translate(
-                            strops.BIBLIST_TRANS).split()
+        tallylist = cr.get('event', 'tallys')
         # append any 'missing' tallys from points data errors
         for i in tallyset:
             if i not in tallylist:
-                self.log.debug('Adding missing tally to config: ' + repr(i))
+                self.log.debug(u'Adding missing tally to config: ' + repr(i))
                 tallylist.append(i)
         # then scan for meta data
         for i in tallylist:
@@ -277,39 +276,34 @@ class rms(object):
                 self.tallymap[i] = {}
                 self.points[i] = {}	 # redundant, but ok
                 self.pointscb[i] = {}
-                crkey = 'tally_' + i
-                descr = ''
-                if cr.has_option(crkey, 'descr'):
-                    descr = cr.get(crkey, 'descr')
-                self.tallymap[i]['descr'] = descr
+                crkey = u'tally_' + i
+                descr = u''
+                if cr.has_option(crkey, u'descr'):
+                    descr = cr.get(crkey, u'descr')
+                self.tallymap[i][u'descr'] = descr
                 keepdnf = False
-                if cr.has_option(crkey, 'keepdnf'):
-                    keepdnf = strops.confopt_bool(cr.get(crkey, 'keepdnf'))
-                self.tallymap[i]['keepdnf'] = keepdnf
+                if cr.has_option(crkey, u'keepdnf'):
+                    keepdnf = strops.confopt_bool(cr.get(crkey, u'keepdnf'))
+                self.tallymap[i][u'keepdnf'] = keepdnf
             else:
-                self.log.info('Ignoring duplicate points tally: ' + repr(i))
+                self.log.info(u'Ignoring duplicate points tally: ' + repr(i))
 
 
-        starters = cr.get('event', 'startlist').split()
-        dolaps = not strops.confopt_bool(cr.get('event', 'nolaps'))
-        #if strops.confopt_bool(cr.get('event', 'resort')):
-            #starters.sort(cmp=lambda x,y: cmp(int(x), int(y)))
+        starters = cr.get(u'event', u'startlist').split()
+        dolaps = not strops.confopt_bool(cr.get(u'event', u'nolaps'))
         for r in starters:
             self.addrider(r)
-            if cr.has_option('riders', r):
+            if cr.has_option(u'riders', r):
                 nr = self.getrider(r)
                 # bib = comment,in,laps,rftod,mbunch,rfseen...
-                ril = csv.reader([cr.get('riders', r)]).next()
+                ril = cr.get(u'riders', r)	# rider op is vec
                 lr = len(ril)
                 if lr > 0:
                     nr[COL_COMMENT] = ril[0]
                 if lr > 1:
                     nr[COL_INRACE] = strops.confopt_bool(ril[1])
                 if lr > 2:
-                    if ril[2].isdigit():
-                        nr[COL_LAPS] = int(ril[2])
-                    else:
-                        nr[COL_LAPS] = 0
+                    nr[COL_LAPS] = strops.confopt_posint(ril[2])
                 if lr > 3:
                     nr[COL_RFTIME] = tod.str2tod(ril[3])
                 if lr > 4:
@@ -322,21 +316,20 @@ class rms(object):
                         if laptod is not None:
                             nr[COL_RFSEEN].append(laptod)
             # record any extra bonus/penalty to rider model
-            if cr.has_option('stagebonus', r):
-                nr[COL_BONUS] = tod.str2tod(cr.get('stagebonus', r))
-            if cr.has_option('stagepenalty', r):
-                nr[COL_PENALTY] = tod.str2tod(cr.get('stagepenalty', r))
-        self.set_start(cr.get('event', 'start'), cr.get('event', 'lstart'))
-        self.set_finish(cr.get('event', 'finish'))
-        self.places = strops.reformat_placelist(cr.get('event', 'places'))
-        self.comment = cr.get('event', 'comment').splitlines()
-        if cr.get('event', 'finished') == 'Yes':
+            if cr.has_option(u'stagebonus', r):
+                nr[COL_BONUS] = tod.str2tod(cr.get(u'stagebonus', r))
+            if cr.has_option(u'stagepenalty', r):
+                nr[COL_PENALTY] = tod.str2tod(cr.get(u'stagepenalty', r))
+        self.set_start(cr.get(u'event', u'start'),
+                       cr.get(u'event', u'lstart'))
+        self.set_finish(cr.get(u'event', u'finish'))
+        self.places = strops.reformat_placelist(cr.get(u'event', u'places'))
+        self.comment = cr.get(u'event', u'comment')	# comments are vec
+        if strops.confopt_bool(cr.get(u'event', u'finished')):
             self.set_finished()
         self.recalculate()
 
- 
-
-        for col in cr.get('event', 'hidecols').split():
+        for col in cr.get(u'event', u'hidecols'):
             target = strops.confopt_posint(col)
             if target is not None:
                 tc = self.view.get_column(target)
@@ -347,10 +340,10 @@ class rms(object):
         # an error message is left on top of status stack. This is not
         # always a hard fail and the user should be left to determine
         # an appropriate outcome.
-        eid = cr.get('event', 'id')
+        eid = cr.get(u'event', u'id')
         if eid and eid != EVENT_ID:
-            self.log.error('Event configuration mismatch: '
-                           + repr(eid) + ' != ' + repr(EVENT_ID))
+            self.log.error(u'Event configuration mismatch: '
+                           + repr(eid) + u' != ' + repr(EVENT_ID))
             self.readonly = True
 
     def get_ridercmdorder(self):
@@ -405,12 +398,12 @@ class rms(object):
             self.canundo = False
           
     def get_catlist(self):
-        """Return the ordered list of categories as a string."""
+        """Return the ordered list of categories."""
         rvec = []
         for cat in self.cats:
             if cat != '':
                 rvec.append(cat)
-        return ','.join(rvec)
+        return rvec
 
     def ridercat(self, cat):
         """Return a category from the result for the riders cat."""
@@ -423,63 +416,58 @@ class rms(object):
     def saveconfig(self):
         """Save event config to disk."""
         if self.readonly:
-            self.log.error('Attempt to save readonly ob.')
+            self.log.error(u'Attempt to save readonly ob.')
             return
-        cw = ConfigParser.ConfigParser()
-        cw.add_section('event')
+        cw = jsonconfig.config()
+        cw.add_section(u'event')
         if self.start is not None:
-            cw.set('event', 'start', self.start.rawtime())
+            cw.set(u'event', u'start', self.start.rawtime())
         if self.lstart is not None:
-            cw.set('event', 'lstart', self.lstart.rawtime())
+            cw.set(u'event', u'lstart', self.lstart.rawtime())
         if self.finish is not None:
-            cw.set('event', 'finish', self.finish.rawtime())
-        if self.timerstat == 'finished':
-            cw.set('event', 'finished', 'Yes')
-        else:
-            cw.set('event', 'finished', 'No')
-        cw.set('event', 'places', self.places)
+            cw.set(u'event', u'finish', self.finish.rawtime())
+        cw.set(u'event', u'finished', self.timerstat == 'finished')
+        cw.set(u'event', u'places', self.places)
 
         # save intermediate data
-        cw.set('event', 'intermeds', ' '.join(self.intermeds))
+        cw.set(u'event', u'intermeds', self.intermeds)
         for i in self.intermeds:
-            crkey = 'intermed_' + i
+            crkey = u'intermed_' + i
             cw.add_section(crkey)
-            cw.set(crkey, 'descr', self.intermap[i]['descr'])
-            cw.set(crkey, 'places', self.intermap[i]['places'])
+            cw.set(crkey, u'descr', self.intermap[i][u'descr'])
+            cw.set(crkey, u'places', self.intermap[i][u'places'])
 
         # save contest meta data
-        cw.set('event', 'contests', ' '.join(self.contests))
+        cw.set(u'event', u'contests', self.contests)
         for i in self.contests:
-            crkey = 'contest_' + i
+            crkey = u'contest_' + i
             cw.add_section(crkey)
-            cw.set(crkey, 'tally', self.contestmap[i]['tally'])
-            cw.set(crkey, 'source', self.contestmap[i]['source'])
-            cw.set(crkey, 'all_source', self.contestmap[i]['all_source'])
+            cw.set(crkey, u'tally', self.contestmap[i][u'tally'])
+            cw.set(crkey, u'source', self.contestmap[i][u'source'])
+            cw.set(crkey, u'all_source', self.contestmap[i][u'all_source'])
             blist = []
-            for b in self.contestmap[i]['bonuses']:
+            for b in self.contestmap[i][u'bonuses']:
                 blist.append(b.rawtime(0))
-            cw.set(crkey, 'bonuses', ' '.join(blist))
+            cw.set(crkey, u'bonuses', u' '.join(blist))
             plist = []
-            for p in self.contestmap[i]['points']:
+            for p in self.contestmap[i][u'points']:
                 plist.append(str(p))
-            cw.set(crkey, 'points', ' '.join(plist))
-
+            cw.set(crkey, u'points', u' '.join(plist))
         # save tally meta data
-        cw.set('event', 'tallys', ' '.join(self.tallys))
+        cw.set(u'event', u'tallys', self.tallys)
         for i in self.tallys:
-            crkey = 'tally_' + i
+            crkey = u'tally_' + i
             cw.add_section(crkey)
-            cw.set(crkey, 'descr', self.tallymap[i]['descr'])
-            cw.set(crkey, 'keepdnf', self.tallymap[i]['keepdnf'])
+            cw.set(crkey, u'descr', self.tallymap[i][u'descr'])
+            cw.set(crkey, u'keepdnf', self.tallymap[i][u'keepdnf'])
 
         # save riders
-        cw.set('event', 'startlist', self.get_startlist())    
+        cw.set(u'event', u'startlist', self.get_startlist())    
         if self.autocats:
-            cw.set('event', 'categories', 'AUTO')
+            cw.set(u'event', u'categories', [u'AUTO'])
         else:
-            cw.set('event', 'categories', self.get_catlist())
-        cw.set('event', 'comment', '\n'.join(self.comment))
-
+            cw.set(u'event', u'categories', self.get_catlist())
+        cw.set(u'event', u'comment', self.comment)
         hides = []
         idx = 0
         for c in self.view.get_columns():
@@ -487,38 +475,35 @@ class rms(object):
                 hides.append(str(idx))
             idx += 1
         if len(hides) > 0:
-            cw.set('event', 'hidecols', ' '.join(hides))
+            cw.set(u'event', u'hidecols', hides)
             
-        cw.add_section('riders')
+        cw.add_section(u'riders')
         # sections for commissaire awarded bonus/penalty
-        cw.add_section('stagebonus')
-        cw.add_section('stagepenalty')
+        cw.add_section(u'stagebonus')
+        cw.add_section(u'stagepenalty')
         for r in self.riders:
-            bf = ''
-            if r[COL_INRACE]:
-                bf='True'
-            rt = ''
+            rt = u''
             if r[COL_RFTIME] is not None:
                 rt = r[COL_RFTIME].rawtime(2)
-            mb = ''
+            mb = u''
             if r[COL_MBUNCH] is not None:
                 mb = r[COL_MBUNCH].rawtime(0)
-            sto = ''
+            sto = u''
             if r[COL_STOFT] is not None:
                 sto = r[COL_STOFT].rawtime(2)
             # bib = comment,in,laps,rftod,mbunch,rfseen...
-            slice = [r[COL_COMMENT], bf, r[COL_LAPS], rt, mb, sto]
+            slice = [r[COL_COMMENT].decode('utf-8'), r[COL_INRACE],
+                     r[COL_LAPS], rt, mb, sto]
             for t in r[COL_RFSEEN]:
                 if t is not None:
                     slice.append(t.rawtime(2))
-            cw.set('riders', r[COL_BIB],
-                    ','.join(map(lambda i: str(i).replace(',', '\\,'), slice)))
+            cw.set(u'riders', r[COL_BIB].decode('utf-8'), slice)
             if r[COL_BONUS] is not None:
-                cw.set('stagebonus', r[COL_BIB], r[COL_BONUS].rawtime(0))
+                cw.set(u'stagebonus', r[COL_BIB], r[COL_BONUS].rawtime(0))
             if r[COL_PENALTY] is not None:
-                cw.set('stagepenalty', r[COL_BIB], r[COL_PENALTY].rawtime(0))
-        cw.set('event', 'id', EVENT_ID)
-        self.log.debug('Saving config to: ' + self.configpath)
+                cw.set(u'stagepenalty', r[COL_BIB], r[COL_PENALTY].rawtime(0))
+        cw.set(u'event', u'id', EVENT_ID)
+        self.log.debug(u'Saving config to: ' + repr(self.configpath))
         with open(self.configpath, 'wb') as f:
             cw.write(f)
 
@@ -609,28 +594,55 @@ class rms(object):
     def startlist_report(self):
         """Return a startlist report."""
         ret = []
+        if len(self.cats) > 1:
+            for c in self.cats:
+                if c:
+                    ret.extend(self.startlist_report_gen(c))
+        else:
+            ret = self.startlist_report_gen()
+        return ret
+
+    def startlist_report_gen(self, cat=None):
+        catname = u''
+        subhead = u''
+        if cat is not None:
+           dbr = self.meet.rdb.getrider(cat,u'cat')
+           if dbr is not None:
+               catname = self.meet.rdb.getvalue(dbr,
+                                         riderdb.COL_FIRST)
+               subhead = self.meet.rdb.getvalue(dbr,
+                                         riderdb.COL_LAST)
+        ret = []
         sec = printing.section()
         sec.heading = 'Startlist'
+        if catname:
+            sec.heading += u': ' + catname
+            sec.subheading = subhead
         cnt = self.reorder_startlist()
+        rcnt = 0
+        cat = self.ridercat(cat)
         for r in self.riders:
-            ucicode = None
-            dbr = self.meet.rdb.getrider(r[COL_BIB],self.series)
-            if dbr is not None:
-                ucicode = self.meet.rdb.getvalue(dbr,
-                                         riderdb.COL_UCICODE)
-            comment = None
-            if r[COL_CAT] == 'U23':
-                comment = '*' 
-            if not r[COL_INRACE]:
-                comment = r[COL_COMMENT]
-                if comment == '':
-                    comment = 'dnf'
-            sec.lines.append([comment, r[COL_BIB], r[COL_NAMESTR],
-                                       ucicode])
+            if cat is None or self.ridercat(r[COL_CAT]) == cat:
+                ucicode = None
+                dbr = self.meet.rdb.getrider(r[COL_BIB],self.series)
+                if dbr is not None:
+                    ucicode = self.meet.rdb.getvalue(dbr,
+                                             riderdb.COL_UCICODE)
+                comment = None
+                if r[COL_CAT] == 'U23':
+                    comment = '*' 
+                if not r[COL_INRACE]:
+                    comment = r[COL_COMMENT].decode('utf-8')
+                    if comment == '':
+                        comment = 'dnf'
+                sec.lines.append([comment, r[COL_BIB].decode('utf-8'),
+                                       r[COL_NAMESTR].decode('utf-8'),
+                                           ucicode])
+                rcnt += 1
         ret.append(sec)
-        if cnt > 1:
-            sec = printing.section()
-            sec.lines.append([None, None, 'Total riders: ' + str(cnt)])
+        if rcnt > 1:
+            sec = printing.bullet_text()
+            sec.lines.append([u'', 'Total riders: ' + unicode(rcnt)])
             ret.append(sec)
         return ret
 
@@ -732,85 +744,92 @@ class rms(object):
                 ret.append(sec)
         else:
             # nothing to report...
-            sec.append([None,None,'-- Not Started --'])
+            pass
         return ret
 
     def catresult_report(self):
         """Return a categorised race result report."""
         self.recalculate()
         ret = []
-        firstcat = True
         for cat in self.cats:
-            if not firstcat:
-                ret.append('')
-            firstcat = False
-            # Don't do this since # riders is not known in this hackmode
-            #catstr = cat
-            #if cat == '':
-                #catstr = 'UNCATEGORISED'
-            ret.append(cat)
+            if not cat:
+                continue	# ignore empty and None cat
+            catname = cat	# fallback emergency
+            subhead = u''
+            dbr = self.meet.rdb.getrider(cat,u'cat')
+            if dbr is not None:
+                catname = self.meet.rdb.getvalue(dbr,
+                                          riderdb.COL_FIRST) # already decode
+                subhead = self.meet.rdb.getvalue(dbr,
+                                          riderdb.COL_LAST)
+            sec = printing.section()
+            sec.heading = 'Result'
+            if catname:
+                sec.heading += u': ' + catname
+                sec.subheading = subhead
             wt = None
             lt = None
             first = True
             lcomment = ''
             lp = None
+            rcnt = 0
             plcnt = 1
             for r in self.riders:
                 rcat = self.ridercat(r[COL_CAT])
                 if cat == rcat:
-                    bstr = r[COL_BIB].rjust(3)
-                    nstr = strops.truncpad(r[COL_NAMESTR], 47)
-                    #cstr = strops.truncpad(r[COL_CAT], 8, align='l', elipsis=False)
-                    pstr = '    '
-                    tstr = '        '
-                    dstr = '        '
+                    bstr = r[COL_BIB].decode('utf-8')
+                    nstr = r[COL_NAMESTR].decode('utf-8')
+                    pstr = u''
+                    tstr = u''
+                    dstr = u''
                     if r[COL_INRACE]:
-                        if r[COL_PLACE] != '':
+                        if r[COL_PLACE] != u'':
                             if lp != r[COL_PLACE]:
-                                lp = str(plcnt)
+                                lp = unicode(plcnt)
                         else:
-                            lp = ''
+                            lp = u''
                         plcnt += 1
-                        pstr = '    '
-                        if lp is not None and lp != '':
-                            pstr = (lp + '.').ljust(4)
+                        pstr = u''
+                        if lp is not None and lp != u'':
+                            pstr = lp + u'.'
                         bt = self.vbunch(r[COL_CBUNCH], r[COL_MBUNCH])
                         if bt is not None:
-                            tstr = bt.rawtime(0).rjust(8)
+                            tstr = bt.rawtime(0)
                             if bt != lt:
                                 if not first:
                                     #ret.append('')	# new bunch
-                                    dstr = ('+' + (bt - wt).rawtime(0)).rjust(8)
+                                    dstr = u'+' + (bt - wt).rawtime(0)
                             if wt is None:	# first finish time
                                 wt = bt
                             first = False
                             # compute elapsed
                             et = bt
-                            if r[COL_STOFT] is not None:	# apply a start offset
-                                dofastest = True	# will need to report!
+                            if r[COL_STOFT] is not None: # apply a start offset
                                 et = bt - r[COL_STOFT]
-                                tstr = et.rawtime(0).rjust(8)
-                            #if fastest is None or et < fastest:
-                                #fastest = et
-                                #fastestbib = r[COL_BIB]
+                                tstr = et.rawtime(0)
                         lt = bt
                     else:
                         comment = r[COL_COMMENT]
-                        if comment == '':
-                            comment = 'dnf'	# override empty comment
+                        if comment == u'':
+                            comment = u'dnf'	# override empty comment
                         if comment != lcomment:
-                            ret.append('')
+                            sec.lines.append([])	# add empty row
                         lcomment = comment
-                        pstr = strops.truncpad(comment, 4)
-                    ret.append(' '.join([pstr, bstr, nstr, tstr, dstr]))
+                        pstr = comment
+                    sec.lines.append([pstr, bstr, nstr, None, tstr, dstr])
+                    rcnt += 1
                 else:
                     # not in this category.
                     pass
-             
+            ret.append(sec)
+            ## TODO: add section counts here
         return ret
 
     def result_report(self):
         """Return a race result report."""
+        if len(self.cats) > 1:
+            return self.catresult_report()	# hack mode for baw baw
+
         self.recalculate()
         ret = []
         wt = None
@@ -980,9 +999,6 @@ class rms(object):
                 for bib in rlist.split():
                     self.query_rider(bib)
             return True
-        elif acode == 'nst':
-            self.new_start(rlist)
-            return True
         elif acode == 'com':
             self.add_comment(rlist)
             return True
@@ -1335,7 +1351,6 @@ class rms(object):
         self.log.debug('Event shutdown: ' + msg)
         if not self.readonly:
             self.saveconfig()
-        self.meet.edb.editevent(self.event, winopen=False)
         self.winopen = False
 
     def starttrig(self, e):
@@ -2157,8 +2172,8 @@ class rms(object):
     def __init__(self, meet, event, ui=True):
         self.meet = meet
         self.event = event      # Note: now a treerowref
-        self.evno = meet.edb.getvalue(event, eventdb.COL_EVNO)
-        self.series = meet.edb.getvalue(event, eventdb.COL_SERIES)
+        self.evno = event[u'evid']
+        self.series = event[u'seri']
         self.configpath = meet.event_configfile(self.evno)
 
         self.log = logging.getLogger('roadrace')
