@@ -35,10 +35,9 @@ import glib
 import gobject
 import os
 import logging
-import csv
-import ConfigParser
 
 import metarace
+from metarace import jsonconfig
 from metarace import tod
 from metarace import eventdb
 from metarace import riderdb
@@ -99,39 +98,42 @@ class sportif(object):
         """Load race config from disk."""
         self.riders.clear()
         self.resettimer()
-        cr = ConfigParser.ConfigParser({'start':'',
-                                        'lstart':'',
-                                        'id':EVENT_ID,
-                                        'finish':'',
-                                        'categories':'',
-                                        'minimums':'',
-                                        'maximums':'',
-                                        'finished':'No',
-                                        'startlist':''})
-        cr.add_section('event')
-        cr.add_section('riders')
-        if os.path.isfile(self.configpath):
-            self.log.debug('Attempting to read config from path='
-                            + repr(self.configpath))
-            cr.read(self.configpath)
+        cr = jsonconfig.config({u'event':{
+                                          u'start':u'',
+                                          u'lstart':u'',
+                                          u'finish':u'',
+                                          u'categories':[],
+                                          u'minimums':[],
+                                          u'maximums':[],
+                                          u'finished':False,
+                                          u'id':EVENT_ID,
+                                          u'startlist':[]}})
+        cr.add_section(u'event')
+        cr.add_section(u'riders')
+        # check for config file
+        try:
+            with open(self.configpath, 'rb') as f:
+                cr.read(f)
+        except Exception as e:
+            self.log.error(u'Reading event config: ' + unicode(e))
 
         # load _result_ categories
-        catlist = cr.get('event', 'categories')
+        catlist = cr.get(u'event', u'categories')
         self.cats = []
-        for cat in catlist.split(','):
+        for cat in catlist:
             if cat != '':
                 cat = cat.strip().upper()
                 self.cats.append(cat)
-        self.cats.append('')    # always include one empty cat
+        self.cats.append(u'')    # always include one empty cat
         minlist = []
-        for mtm in cr.get('event', 'minimums').split(','):
+        for mtm in cr.get(u'event', u'minimums'):
             minlist.append(tod.str2tod(mtm))
         while len(minlist) < len(self.cats):
             minlist.append(None)	# extend as necessary
 
         # and repead for maxlist
         maxlist = []
-        for mtm in cr.get('event', 'maximums').split(','):
+        for mtm in cr.get(u'event', u'maximums'):
             maxlist.append(tod.str2tod(mtm))
         while len(maxlist) < len(self.cats):
             maxlist.append(None)	# extend as necessary
@@ -143,14 +145,14 @@ class sportif(object):
             self.minmap[self.cats[j]] = minlist[j]
             self.maxmap[self.cats[j]] = maxlist[j]
 
-        starters = cr.get('event', 'startlist').split()
+        starters = cr.get(u'event', u'startlist')
         # for a sportif - always sort by bib
         for r in starters:
             self.addrider(r)
-            if cr.has_option('riders', r):
+            if cr.has_option(u'riders', r):
                 nr = self.getrider(r)
+                ril = cr.get(u'riders', r)      # rider op is vec
                 # bib = comment,rftod,rfseen...
-                ril = csv.reader([cr.get('riders', r)]).next()
                 lr = len(ril)
                 if lr > 0:
                     nr[COL_COMMENT] = ril[0]
@@ -163,18 +165,18 @@ class sportif(object):
                         laptod = tod.str2tod(ril[i])
                         if laptod is not None:
                             nr[COL_RFSEEN].append(laptod)
-        self.set_start(cr.get('event', 'start'), cr.get('event', 'lstart'))
-        self.set_finish(cr.get('event', 'finish'))
-        if cr.get('event', 'finished') == 'Yes':
+        self.set_start(cr.get(u'event', u'start'), cr.get(u'event', u'lstart'))
+        self.set_finish(cr.get(u'event', u'finish'))
+        if strops.confopt_bool(cr.get(u'event', u'finished')):
             self.set_finished()
 
         # After load complete - check config and report. This ensures
         # an error message is left on top of status stack. This is not
         # always a hard fail and the user should be left to determine
         # an appropriate outcome.
-        eid = cr.get('event', 'id')
+        eid = cr.get(u'event', u'id')
         if eid != EVENT_ID:
-            self.log.error('Event configuration mismatch: '
+            self.log.error(u'Event configuration mismatch: '
                            + repr(eid) + ' != ' + repr(EVENT_ID))
 
     def get_ridercmdorder(self):
@@ -186,30 +188,27 @@ class sportif(object):
         return RIDER_COMMANDS
 
     def get_startlist(self):
-        """Return a list of all rider numbers 'registered' to event."""
+        """Return a list of riders 'registered' to event."""
         ret = []
         for r in self.riders:
             ret.append(r[COL_BIB])
-        return ' '.join(ret)
+        return ret
 
     def saveconfig(self):
         """Save event config to disk."""
         if self.readonly:
-            self.log.error('Attempt to save readonly ob.')
+            self.log.error(u'Attempt to save readonly ob.')
             return
-        cw = ConfigParser.ConfigParser()
-        cw.add_section('event')
+        cw = jsonconfig.config()
+        cw.add_section(u'event')
         if self.start is not None:
-            cw.set('event', 'start', self.start.rawtime())
+            cw.set(u'event', u'start', self.start.rawtime())
         if self.lstart is not None:
-            cw.set('event', 'lstart', self.lstart.rawtime())
+            cw.set(u'event', u'lstart', self.lstart.rawtime())
         if self.finish is not None:
-            cw.set('event', 'finish', self.finish.rawtime())
-        if self.timerstat == 'finished':
-            cw.set('event', 'finished', 'Yes')
-        else:
-            cw.set('event', 'finished', 'No')
-        cw.set('event', 'startlist', self.get_startlist())    
+            cw.set(u'event', u'finish', self.finish.rawtime())
+        cw.set(u'event', u'finished', self.timerstat == 'finished')
+        cw.set(u'event', u'startlist', self.get_startlist())    
         catlist = []
         minlist = []
         maxlist = []
@@ -220,33 +219,32 @@ class sportif(object):
                 if mtm is not None:
                     minlist.append(mtm.rawtime())
                 else:
-                    minlist.append('')
+                    minlist.append(u'')
                 mtm = self.maxmap[cat]
                 if mtm is not None:
                     maxlist.append(mtm.rawtime())
                 else:
-                    maxlist.append('')
-        cw.set('event', 'categories', ','.join(catlist))
-        cw.set('event', 'minimums', ','.join(minlist))
-        cw.set('event', 'maximums', ','.join(maxlist))
+                    maxlist.append(u'')
+        cw.set(u'event', u'categories', catlist)
+        cw.set(u'event', u'minimums', minlist)
+        cw.set(u'event', u'maximums', maxlist)
 
-        cw.add_section('riders')
+        cw.add_section(u'riders')
         for r in self.riders:
-            st = ''
+            st = u''
             if r[COL_START] is not None:
                 st = r[COL_START].rawtime(2)
-            rt = ''
+            rt = u''
             if r[COL_RFTIME] is not None:
                 rt = r[COL_RFTIME].rawtime(2)
             # bib = comment,rftod,rfseen...
-            slice = [r[COL_COMMENT], st, rt]
+            slice = [r[COL_COMMENT].decode('utf-8'), st, rt]
             for t in r[COL_RFSEEN]:
                 if t is not None:
                     slice.append(t.rawtime(2))
-            cw.set('riders', r[COL_BIB],
-                    ','.join(map(lambda i: str(i).replace(',', '\\,'), slice)))
-        cw.set('event', 'id', EVENT_ID)
-        self.log.debug('Saving config to: ' + self.configpath)
+            cw.set(u'riders', r[COL_BIB].decode('utf-8'), slice)
+        cw.set(u'event', u'id', EVENT_ID)
+        self.log.debug(u'Saving config to: ' + repr(self.configpath))
         with open(self.configpath, 'wb') as f:
             cw.write(f)
 
@@ -264,8 +262,8 @@ class sportif(object):
 
     def set_titlestr(self, titlestr=None):
         """Update the title string label."""
-        if titlestr is None or titlestr == '':
-            titlestr = 'Sportif Ride'
+        if titlestr is None or titlestr == u'':
+            titlestr = u'Sportif Ride'
         self.title_namestr.set_text(titlestr)
 
     def destroy(self):
@@ -292,14 +290,16 @@ class sportif(object):
         """Return a startlist report."""
         ret = []
         sec = printing.section()
-        sec.heading = 'Startlist'
+        sec.heading = u'Startlist'
         cnt = self.reorder_startlist()
         for r in self.riders:
-            sec.lines.append([None, r[COL_BIB], r[COL_NAMESTR], r[COL_CAT]])
+            sec.lines.append([None, r[COL_BIB].decode('utf-8'),
+                                    r[COL_NAMESTR].decode('utf-8'),
+                                    r[COL_CAT].decode('utf-8')])
         ret.append(sec)
         if cnt > 1:
             sec = printing.section()
-            sec.lines.append([None, None, 'Total riders: ' + str(cnt)])
+            sec.lines.append([None, None, u'Total riders: ' + unicode(cnt)])
             ret.append(sec)
         return ret
 
@@ -326,40 +326,46 @@ class sportif(object):
             st = self.start
 
         sec = printing.section()
-        sec.heading = 'Individuals'
+        sec.heading = u'Individuals'
 
         self.reorder_byname()
 
         for r in self.riders:
             et = self.get_rider_elapsed(self.getiter(r[COL_BIB]))
-            cat = self.ridercat(r[COL_CAT])
+            bib = r[COL_BIB].decode('utf-8')
+            cat = self.ridercat(r[COL_CAT].decode('utf-8'))
             if et is not None and cat:
                 if self.minmap[cat] is not None and et < self.minmap[cat]:
-                    self.log.warn('Rider time less than minimum: '
-                                     + repr(r[COL_BIB]))
+                    self.log.warn(u'Rider time less than minimum: '
+                                     + repr(bib))
                     et = None	
                 elif self.maxmap[cat] is not None and et > self.maxmap[cat]:
-                    self.log.warn('Rider time greater than maximum: '
-                                     + repr(r[COL_BIB]))
+                    self.log.warn(u'Rider time greater than maximum: '
+                                     + repr(bib))
                     et = None	
-            estr = ''
+            estr = u''
             if et is not None:
-                if r[COL_TEAM]:
-                    if r[COL_TEAM] not in tmap:
-                        tmap[r[COL_TEAM]] = []	# vec of elapsed times
-                    tmap[r[COL_TEAM]].append(et)
+                team = r[COL_TEAM].decode('utf-8')
+                if team:
+                    if team not in tmap:
+                        tmap[team] = []	# vec of elapsed times
+                    tmap[team].append(et)
                 estr = et.rawtime(0)
+            elif r[COL_COMMENT]:
+                estr = r[COL_COMMENT]
             elif r[COL_START] is not None:
-                estr = '[on course]'
+                estr = u'[on course]'
             else:
-                estr = '[not started]'
+                estr = u'[not started]'
 
-            sec.lines.append([None, r[COL_BIB], r[COL_NAMESTR], r[COL_CAT],
+            sec.lines.append([None, bib,
+                                    r[COL_NAMESTR].decode('utf-8'),
+                                    r[COL_CAT].decode('utf-8'),
                                     estr, None])
         ret.append(sec)
 
         sec = printing.section()
-        sec.heading = 'Teams'
+        sec.heading = u'Teams'
         tres = []
         for team in tmap:
             rvec = tmap[team]
@@ -372,22 +378,23 @@ class sportif(object):
             for res in tres:
                 res[4] = res[4].rawtime(0)
                 sec.lines.append(res)
-        ret.append(sec)
+        if len(sec.lines) > 0:
+            ret.append(sec)
         return ret
 
     def camera_report(self):
         """Return a judges (camera) report."""
-        self.log.error('Judges report not available for sportif rides.')
+        self.log.error(u'Judges report not available for sportif rides.')
         return None
 
     def points_report(self):
         """Return a points tally report."""
-        self.log.error('Points tally report not available for sportif rides.')
+        self.log.error(u'Points tally report not available for sportif rides.')
         return None
 
     def stat_but_clicked(self):
         """Deal with a status button click in the main container."""
-        self.log.debug('Stat button clicked.')
+        self.log.debug(u'Stat button clicked.')
 
     def fmt_rider_result(self, r, st):
         """Return a string for the provided rider reference."""
@@ -395,24 +402,14 @@ class sportif(object):
 # [not started]|
 # [on course]|
 # 12h13:12.2|
-        fstr = ''
+        fstr = u''
         et = self.get_rider_elapsed(self.getiter(r[COL_BIB]))
         if et is not None:
             fstr = et.rawtime(1)
-            #di = self.meet.get_distance()
-            #if di:
-                #fstr += '  {0:.1f} km/h'.format(3600 * float(di) / float(et.timeval))
-            # distance calcs require some thought and cat lookups
         elif r[COL_START] is not None:
-            fstr = '[on course]'
+            fstr = u'[on course]'
         else:
-            fstr = '[not started]'
-        
-        #if len(r[COL_RFSEEN]) > 3:	# ignore intermeds
-            #lt = st
-            #for split in r[COL_RFSEEN]:
-                #ret += ' ' + (split - lt).rawtime(0).rjust(7)
-                #lt = split
+            fstr = u'[not started]'
         return fstr
 
     def query_rider(self, bib=None):
@@ -427,22 +424,24 @@ class sportif(object):
         r = self.getrider(bib)
         if r is not None:
             fstr = self.fmt_rider_result(r, st)
-            self.log.info('Rider ' + repr(bib) + ' : ' + repr(fstr))
-            self.meet.announce_rider(['',bib,r[COL_NAMESTR],
-                                     r[COL_CAT],fstr])
+            self.log.info(u'Rider ' + repr(bib) + u' : ' + repr(fstr))
+            self.meet.announce_rider([u'',bib,
+                                     r[COL_NAMESTR].decode('utf-8'),
+                                     r[COL_CAT].decode('utf-8'),
+                                     fstr])
         else:
-            self.log.info('Rider = ' + repr(bib) + ' not in startlist.')
+            self.log.info(u'Rider = ' + repr(bib) + u' not in startlist.')
 
         return False # allow push via idle_add(...
 
     def add_comment(self, comment=''):
         """Append a race comment."""
-        self.log.info('Add comment: ' + repr(comment))
+        self.log.info(u'Add comment: ' + repr(comment))
 
     def ctrl_change(self, acode='', entry=None):
         """Notify change in action combo."""
         if entry is not None:
-            entry.set_text('')
+            entry.set_text(u'')
 
     def race_ctrl(self, acode='', rlist=''):
         """Apply the selected action to the provided bib list."""
@@ -467,7 +466,7 @@ class sportif(object):
             self.add_comment(rlist)
             return True
         else:
-            self.log.error('Ignoring invalid action.')
+            self.log.error(u'Ignoring invalid action.')
         return False
 
     def startlist_gen(self, cat=''):
@@ -513,13 +512,13 @@ class sportif(object):
 
     def clear_results(self):
         """Clear all data from event model."""
-        self.log.debug('Clear results not implemented.')
+        self.log.debug(u'Clear results not implemented.')
 
     def getrider(self, bib):
         """Return reference to selected rider no."""
         ret = None
         for r in self.riders:
-            if r[COL_BIB] == bib:
+            if r[COL_BIB].decode('utf-8') == bib:
                 ret = r
                 break
         return ret
@@ -528,7 +527,7 @@ class sportif(object):
         """Return temporary iterator to model row."""
         i = self.riders.get_iter_first()
         while i is not None:
-            if self.riders.get_value(i, COL_BIB) == bib:
+            if self.riders.get_value(i, COL_BIB).decode('utf-8') == bib:
                 break
             i = self.riders.iter_next(i)
         return i
@@ -539,10 +538,10 @@ class sportif(object):
         if i is not None:
             self.riders.remove(i)
 
-    def addrider(self, bib=''):
+    def addrider(self, bib=u''):
         """Add specified rider to race model."""
-        if bib == '' or self.getrider(bib) is None:
-            nr = [bib, '', '', '', '', '', None, None, []]
+        if bib == u'' or self.getrider(bib) is None:
+            nr = [bib, u'', u'', u'', u'', u'', None, None, []]
             dbr = self.meet.rdb.getrider(bib, self.series)
             if dbr is not None:
                 first = self.meet.rdb.getvalue(dbr, riderdb.COL_FIRST)
@@ -614,12 +613,11 @@ class sportif(object):
                     return True
         return False
 
-    def shutdown(self, win=None, msg='Race Sutdown'):
+    def shutdown(self, win=None, msg=u'Race Sutdown'):
         """Close event."""
-        self.log.debug('ms shutdown: ' + msg)
+        self.log.debug(u'shutdown: ' + msg)
         if not self.readonly:
             self.saveconfig()
-        self.meet.edb.editevent(self.event, winopen=False)
         self.winopen = False
 
     def starttrig(self, e):
@@ -642,31 +640,32 @@ class sportif(object):
         bib = self.meet.rdb.getvalue(r, riderdb.COL_BIB)
         ser = self.meet.rdb.getvalue(r, riderdb.COL_SERIES)
         if ser != self.series:
-            self.log.error('Ignored non-series rider: ' + bib + '.' + ser)
+            self.log.error(u'Ignored non-series rider: ' + repr(bib)
+                               + u'.' + repr(ser))
             return
 
         lr = self.getrider(bib)
         if lr is None:	# sportif uses relaxed 'club mode'
             self.addrider(bib)
             lr = self.getrider(bib)
-            self.log.info('Added non starter: ' + bib
-                          + ' @ ' + e.rawtime(1))
+            self.log.info(u'Added non starter: ' + repr(bib)
+                          + u' @ ' + e.rawtime(1))
 
         # at this point should always have a valid rider vector
         if self.timerstat == 'armstart':
             if lr[COL_START] is None:
-                self.log.info('Starting: ' + bib + ' @ ' + e.rawtime(1))
+                self.log.info(u'Starting: ' + bib + u' @ ' + e.rawtime(1))
                 lr[COL_START] = e
                 self.__dorecalc = True	# starters flag recalc
             else:
-                self.log.error('Duplicate start rider = ' + bib
-                                  + ' @ ' + str(e))
+                self.log.error(u'Duplicate start rider = ' + repr(bib)
+                                  + u' @ ' + e.rawtime(1))
                 lr[COL_RFSEEN].append(e)
             glib.idle_add(self.query_rider, bib)
         elif self.timerstat not in ['idle', 'finished']:
             # save RF ToD into 'seen' vector and log
             lr[COL_RFSEEN].append(e)
-            self.log.info('Saw: ' + bib + ' @ ' + e.rawtime(1))
+            self.log.info(u'Saw: ' + repr(bib) + u' @ ' + e.rawtime(1))
             # record finish time if required
             if self.timerstat == 'armfinish':
                 if lr[COL_RFTIME] is None:
@@ -674,8 +673,8 @@ class sportif(object):
                     self.__dorecalc = True	# finishers flag recalc
                     glib.idle_add(self.query_rider, bib)
                 else:
-                    self.log.error('Duplicate finish rider = ' + bib
-                                      + ' @ ' + str(e))
+                    self.log.error(u'Duplicate finish rider = ' + repr(bib)
+                                      + u' @ ' + e.rawtime(1))
 
     def recalculate(self):
         """Update any internal structures and trigger export if required."""
@@ -749,7 +748,7 @@ class sportif(object):
 
     def ridercat(self, cat):
         """Return a category from the result for the riders cat."""
-        ret = ''        # default is the 'None' category - uncategorised
+        ret = u''        # default is the 'None' category - uncategorised
         checka = cat.upper()
         if checka in self.cats:
             ret = checka
@@ -781,11 +780,11 @@ class sportif(object):
         ft = ''
         if self.finish is not None:
             ft = self.finish.rawtime(2)
-        (ret, st, ft) = uiutil.edit_times_dlg(self.meet.window, st, ft)
-        if ret == 1:
+        rvec = uiutil.edit_times_dlg(self.meet.window, st, ft)
+        if rvec[0] == 1:
             self.set_start(st)
             self.set_finish(ft)
-            self.log.info('Adjusted race times.')
+            self.log.info(u'Adjusted race times.')
 
     def editcol_cb(self, cell, path, new_text, col):
         """Edit column callback."""
@@ -826,13 +825,13 @@ class sportif(object):
     def __init__(self, meet, event, ui=True):
         self.meet = meet
         self.event = event      # Note: now a treerowref
-        self.evno = meet.edb.getvalue(event, eventdb.COL_EVNO)
-        self.series = meet.edb.getvalue(event, eventdb.COL_SERIES)
+        self.evno = event[u'evid']
+        self.series = event[u'seri']
         self.configpath = meet.event_configfile(self.evno)
 
         self.log = logging.getLogger('sportif')
         self.log.setLevel(logging.DEBUG)
-        self.log.debug('opening event: ' + str(self.evno))
+        self.log.debug(u'opening event: ' + repr(self.evno))
 
         # race run time attributes
         self.readonly = not ui
