@@ -14,256 +14,85 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+from __future__ import (division, print_function, absolute_import)
 
 """Basic string filtering, truncation and padding."""
 
 import re
-import metarace
 import os
+from . import timy	# import timy channel strings
 
+# replace codepoints 0->255 with space unless overridden
+# "protective" against unencoded ascii strings and control chars
+SPACEBLOCK = u''
+for i in xrange(0,256):
+    SPACEBLOCK += unichr(i)
 
-INTEGER_UTRANS = {}	# unicode variant, populated below
-INTEGER_TRANS = '\
-        \
-        \
-        \
-        \
-        \
-     -  \
-01234567\
-89      \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        '
-"""Bib translation table for digit strings."""
+# unicode translation 'map' class
+class unicodetrans:
+  def __init__(self, keep=u'', replace=SPACEBLOCK, replacechar=u' '):
+    self.comp = dict((ord(c),replacechar) for c in replace)
+    for c in keep:
+        self.comp[ord(c)] = c
+  def __getitem__(self, k):	# override to return a None
+    return self.comp.get(k)
 
-PLACELIST_TRANS = '\
-        \
-        \
-        \
-        \
-        \
-     -  \
-01234567\
-89      \
- ABCDEFG\
-HIJKLMNO\
-PQRSTUVW\
-XYZ     \
- abcdefg\
-hijklmno\
-pqrstuvw\
-xyz     \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        '
-"""Bib translation table for place listings."""
+INTEGER_UTRANS = unicodetrans(u'-0123456789')
+PLACELIST_UTRANS = unicodetrans(
+u'-0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz')
+PLACESERLIST_UTRANS = unicodetrans(
+u'-.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz')
+BIBLIST_UTRANS = unicodetrans(
+u'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz')
+BIBSERLIST_UTRANS = unicodetrans(
+u'.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz')
+WEBFILE_UTRANS = unicodetrans(
+u'_0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz',u'.',u'_')
+# special case: map controls and spaces, but keep everything else
+PRINT_UTRANS = {}
+for cp in xrange(0,0x20):
+    PRINT_UTRANS[cp] = u' '
+for cp in xrange(0x7f,0xa1):
+    PRINT_UTRANS[cp] = u' '
+for cp in xrange(0x2000,0x200B):
+    PRINT_UTRANS[cp] = u' '
+PRINT_UTRANS[0x1680] = u' '
+PRINT_UTRANS[0x180e] = u' '
+PRINT_UTRANS[0x202f] = u' '
+PRINT_UTRANS[0x205f] = u' '
+PRINT_UTRANS[0x3000] = u' '
 
-PLACESERLIST_TRANS = '\
-        \
-        \
-        \
-        \
-        \
-     -. \
-01234567\
-89      \
- ABCDEFG\
-HIJKLMNO\
-PQRSTUVW\
-XYZ     \
- abcdefg\
-hijklmno\
-pqrstuvw\
-xyz     \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        '
-"""Bib translation table for place listings with bib.ser strings."""
-
-BIBLIST_TRANS = '\
-        \
-        \
-        \
-        \
-        \
-        \
-01234567\
-89      \
- ABCDEFG\
-HIJKLMNO\
-PQRSTUVW\
-XYZ     \
- abcdefg\
-hijklmno\
-pqrstuvw\
-xyz     \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        '
-"""Bib translation table for parsing bib lists."""
-
-BIBSERLIST_TRANS = '\
-        \
-        \
-        \
-        \
-        \
-      . \
-01234567\
-89      \
- ABCDEFG\
-HIJKLMNO\
-PQRSTUVW\
-XYZ     \
- abcdefg\
-hijklmno\
-pqrstuvw\
-xyz     \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        '
-"""Bib.ser translation table for parsing bib.ser lists."""
-
-PRINT_TRANS = '\
-        \
-        \
-        \
-        \
- !"#$%&\'\
-()*+,-./\
-01234567\
-89:;<=>?\
-@ABCDEFG\
-HIJKLMNO\
-PQRSTUVW\
-XYZ[\\]^_\
-`abcdefg\
-hijklmno\
-pqrstuvw\
-xyz{|}~ \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        \
-        '
-"""Basic printing ASCII character table."""
-
-def img_filename(path, basename=''):
+def img_filename(path, basename=u''):
     """Return the 'best' filename for the given basename image."""
     ret = None
-    srchs = ['svg', 'eps', 'pdf', 'png', 'jpg', 'bmp']
-    basename = basename.rstrip('.')
+    srchs = [u'svg', u'eps', u'pdf', u'png', u'jpg', u'bmp']
+    basename = basename.rstrip(u'.')
     for xtn in srchs:
-        checka = os.path.join(path, basename + '.' + xtn)
+        checka = os.path.join(path, basename + u'.' + xtn)
         if os.path.isfile(checka):
             ret = checka
             break
     return ret
 
-DNFCODEMAP = { 'hd': 0,
-               'dsq': 1,
-               'dnf': 2,
-               'dns': 3,
-               '': 2}
+DNFCODEMAP = { u'hd': 0,
+               u'dsq': 1,
+               u'dnf': 2,
+               u'dns': 3,
+               u'': 2}
 
 def cmp_dnf(x, y):
     """Comparison func for two dnf codes."""
     if x not in DNFCODEMAP:
-        x = ''
+        x = u''
     if y not in DNFCODEMAP:
-        y = ''
+        y = u''
     return cmp(DNFCODEMAP[x], DNFCODEMAP[y])
     
 def riderno_key(bib):
     """Return a comparison key for sorting rider number strings."""
     return bibstr_key(bib)
 
-def bibstr_key(bibstr=''):
+def bibstr_key(bibstr=u''):
     """Return a comparison key for sorting rider bib.ser strings."""
     (bib, ser) = bibstr2bibser(bibstr)
     bval = 0
@@ -272,29 +101,29 @@ def bibstr_key(bibstr=''):
     else:
         bval = id(bib)
     sval = 0
-    if ser != '':
+    if ser != u'':
         sval = ord(ser[0])<<12
     return sval | (bval&0xfff)
 
-def promptstr(prompt='', value=''):
+def promptstr(prompt=u'', value=u''):
     """Prefix a non-empty string with a prompt, or return empty."""
-    ret = ''
+    ret = u''
     if value:
-        ret = prompt + ' ' + value
+        ret = prompt + u' ' + value
     return ret
 
-def listsplit(liststr=''):
+def listsplit(liststr=u''):
     """Return a split and stripped list."""
     ret = []
-    for e in liststr.split(','):
+    for e in liststr.split(u','):
         ret.append(e.strip())
     return ret
 
 def heatsplit(heatstr):
     """Return a failsafe heat/lane pair for the supplied heat string."""
-    hv = heatstr.split('.')
+    hv = heatstr.split(u'.')
     while len(hv) < 2:
-        hv.append('0')
+        hv.append(u'0')
     return(riderno_key(hv[0]), riderno_key(hv[1]))
     
 def fitname(first, last, width, trunc=False):
@@ -304,65 +133,82 @@ def fitname(first, last, width, trunc=False):
 
     1: 'First Lastone-Lasttwo'    - simple concat
     2: 'First Lasttwo'            - ditch hypenated name
-    3: 'F. Lasttwo'               - abbrev first name
-    4: 'F Lasttwo'                - get 1 xtra char omit period
-    5: 'F. Lasttwo'               - give up and return name for truncation
+    3: 'First V Lastname'	  - abbrev Von part
+    4: 'F. Lasttwo'               - abbrev first name
+    5: 'F Lasttwo'                - get 1 xtra char omit period
+    6: 'F. Lasttwo'               - give up and return name for truncation
 
     If optional param trunc is set and field would be longer than
     width, truncate and replace the last 3 chars with elipsis '...'
     Unless only two char longer than field - then just chop final chars
 
     """
-    ret = ''
-    fstr = str(first).strip().title()
-    lstr = str(last).strip().upper()
-    trystr = (fstr + ' ' + lstr).strip()
+    ret = u''
+    fstr = first.strip().title()
+    lstr = last.strip().upper()
+    trystr = (fstr + u' ' + lstr).strip()
     if len(trystr) > width:
-        lstr = lstr.split('-')[-1].strip()
-        trystr = fstr + ' ' + lstr
+        lstr = lstr.split('-')[-1].strip()	# Hyphen
+        trystr = fstr + u' ' + lstr
         if len(trystr) > width:
-            if len(fstr) > 0:
-                trystr = fstr[0] + '. ' + lstr
-            else:
-                trystr = lstr
-            if len(trystr) == width + 1 and len(fstr) > 0:  # opportunistic
-                trystr = fstr[0] + ' ' + lstr
+            lstr = lstr.replace(u'VON ',u'V ')	# Von part
+            lstr = lstr.replace(u'VAN ',u'V ')
+            trystr = fstr + u' ' + lstr
+            if len(trystr) > width:
+                if len(fstr) > 0:		# initial first name
+                    trystr = fstr[0] + u'. ' + lstr
+                else:
+                    trystr = lstr
+                if len(trystr) == width + 1 and len(fstr) > 0:  # opportunistic
+                    trystr = fstr[0] + u' ' + lstr
     if trunc:
         ret = trystr[0:width]
         if width > 6:
             if len(trystr) > width+2:
-                ret = trystr[0:(width - 3)] + '...'
+                ret = trystr[0:(width - 3)] + u'...'
     else:
         ret = trystr
     return ret
 
+def drawno_encirc(drawstr=u''):
+    ret = drawstr
+    if drawstr.isdigit():	# can toint
+        try:
+            ival = int(drawstr)
+            if ival > 0 and ival <= 10:
+                ret = (u'\u00a0' + 	# hack to get full line height?
+                       unichr(0x245f + ival)) # CP U+2460 "Circled digit"
+        except:
+            pass
+    return ret
+
 def num2ord(place):
     """Return ordinal for the given place."""
-    omap = { '1' : 'st',
-             '2' : 'nd',
-             '3' : 'rd',
-             '11' : 'th',
-             '12' : 'th',
-             '13' : 'th' }
+    omap = { u'1' : u'st',
+             u'2' : u'nd',
+             u'3' : u'rd',
+             u'11' : u'th',
+             u'12' : u'th',
+             u'13' : u'th' }
     if place in omap:
         return place + omap[place]
     elif place.isdigit():
-        if len(place) > 1 and place[-1] in omap:
+        if len(place) > 1 and place[-1] in omap: # last digit 1,2,3
             return place + omap[place[-1]]
         else:
-            return place + 'th'
+            return place + u'th'
     else:
         return place
 
 def mark2int(handicap):
     """Convert a handicap string into an integer number of metres."""
-    handicap = handicap.strip().lower()
+    handicap = handicap.decode('utf-8','replace').strip().lower()
     ret = None				# not recognised as handicap
-    if handicap != '':
-        if handicap[0:3] == 'scr':		# 'scr{atch}'
+    if handicap != u'':
+        if handicap[0:3] == u'scr':		# 'scr{atch}'
             ret = 0
         else:				# try [number]m form
-           handicap = handicap.translate(INTEGER_TRANS).strip()
+           handicap = handicap.translate(INTEGER_UTRANS).strip()
            try:
                ret = int(handicap)
            except:
@@ -374,7 +220,7 @@ def truncpad(srcline, length, align='l', elipsis=True):
     ret = srcline[0:length]
     if length > 6:
         if len(srcline) > length+2 and elipsis:
-            ret = srcline[0:(length - 3)] + '...'
+            ret = srcline[0:(length - 3)] + u'...'	# repl with elipsis?
     if align == 'l':
         ret = ret.ljust(length)
     elif align == 'r':
@@ -384,75 +230,76 @@ def truncpad(srcline, length, align='l', elipsis=True):
     return ret
 
 def search_name(namestr):
-    return namestr.translate(BIBLIST_TRANS).strip().lower()
+    return namestr.translate(BIBLIST_UTRANS).strip().lower()
 
 def resname_bib(bib, first, last, club):
     """Return rider name formatted for results with bib (champs/live)."""
-    ret = bib + ' ' + fitname(first, last, 64)
-    if club is not None and club != '':
-        ret += ' (' + club + ')'
+    ret = bib + u' ' + fitname(first, last, 64)
+    if club is not None and club != u'':
+        ret += u' (' + club + u')'
     return ret
 
 def resname(first, last, club):
     """Return rider name formatted for results."""
     ret = fitname(first, last, 64)
-    if club is not None and club != '':
-        ret += ' (' + club + ')'
+    if club is not None and club != u'':
+        ret += u' (' + club + u')'
     return ret
 
 def listname(first, last=None, club=None):
     """Return a rider name summary field for non-edit lists."""
     ret = fitname(first, last, 32)
     if club:
-        ret += ' (' + club + ')'
+        ret += u' (' + club + u')'
     return ret
 
 def reformat_bibserlist(bibserstr):
     """Filter and return a bib.ser start list."""
-    return ' '.join(bibserstr.translate(BIBSERLIST_TRANS).split())
+    return u' '.join(bibserstr.decode('utf-8','replace').translate(BIBSERLIST_UTRANS).split())
 
 def reformat_bibserplacelist(placestr):
     """Filter and return a canonically formatted bib.ser place list."""
-    if placestr.find('-') < 0:		# This is the 'normal' case!
+    placestr = placestr.decode('utf-8', 'replace')
+    if u'-' not in placestr:		# This is the 'normal' case!
         return reformat_bibserlist(placestr)
     # otherwise, do the hard substitutions...
     # TODO: allow the '=' token to indicate RFPLACES ok 
-    placestr = placestr.translate(PLACESERLIST_TRANS).strip()
+    placestr = placestr.translate(PLACESERLIST_UTRANS).strip()
     placestr = re.sub(r'\s*\-\s*', r'-', placestr)	# remove surrounds
     placestr = re.sub(r'\-+', r'-', placestr)		# combine dupes
-    return ' '.join(placestr.strip('-').split())
+    return u' '.join(placestr.strip(u'-').split())
 
 def reformat_biblist(bibstr):
     """Filter and return a canonically formatted start list."""
-    return ' '.join(bibstr.translate(BIBLIST_TRANS).split())
+    return u' '.join(bibstr.decode('utf-8','replace').translate(BIBLIST_UTRANS).split())
 
-def reformat_riderlist(riderstr, rdb=None, series=''):
+def reformat_riderlist(riderstr, rdb=None, series=u''):
     """Filter, search and return a list of matching riders for entry."""
-    ret = ''
+    ret = u''
     ##riderstr = riderstr.translate(PLACELIST_TRANS).lower()
-    riderstr = riderstr.lower()
+    riderstr = riderstr.decode('utf-8', 'replace').lower()
 
     # special case: 'all' -> return all riders from the sepcified series.
-    if riderstr.strip() == 'all':
+    if riderstr.strip() == u'all':
         for cat in rdb.listcats(series):
-            ret += ' ' + rdb.biblistfromcat(cat, series)
-            riderstr = ''
+            ret += u' ' + rdb.biblistfromcat(cat, series)
+            riderstr = u''
     
     # pass 1: search for categories
     if rdb is not None:
         for cat in sorted(rdb.listcats(series), key=len, reverse=True):
-            if len(cat) > 0 and riderstr.find(cat.lower()) >= 0:
-                ret += ' ' + rdb.biblistfromcat(cat, series)
-                riderstr = riderstr.replace(cat.lower(), '')
+            if len(cat) > 0 and cat.lower() in riderstr:
+                ret += u' ' + rdb.biblistfromcat(cat, series)
+                riderstr = riderstr.replace(cat.lower(), u'')
 
     # pass 2: append riders and expand any series if possible
     riderstr = reformat_placelist(riderstr)
     for nr in riderstr.split():
-        if nr.find('-') >= 0:
+        if u'-' in nr:
             # try for a range...
             l = None
             n = None
-            for r in nr.split('-'):
+            for r in nr.split(u'-'):
                 if l is not None:
                     if l.isdigit() and r.isdigit():
                         start = int(l)
@@ -460,36 +307,43 @@ def reformat_riderlist(riderstr, rdb=None, series=''):
                         if start < end:
                             c = start
                             while c < end:
-                                ret += ' ' + str(c)
+                                ret += u' ' + unicode(c)
                                 c += 1
                         else:
-                            ret += ' ' + l	# give up on last val
+                            ret += u' ' + l	# give up on last val
                     else:
                         # one or both not ints
-                        ret += ' ' + l
+                        ret += u' ' + l
                 else:
                     pass
                 l = r
             if l is not None: # catch final value
-                ret += ' ' + l
+                ret += u' ' + l
         else:
-            ret += ' ' + nr
+            ret += u' ' + nr
     # pass 3: reorder and join for return
     rvec = list(set(ret.split()))
     rvec.sort(key=riderno_key)
-    return ' '.join(rvec)
+    return u' '.join(rvec)
 
-def placeset(spec=''):
-    """Convert a place spec into a set of place ints."""
-    ret = ''
+def placeset(spec=u''):
+    """Convert a place spec into an ordered set of place ints."""
+
+    # NOTE: ordering of the set must be retained to correctly handle
+    #       autospecs where the order of the places is not increasing
+    #       eg: sprint semi -> sprint final, the auto spec is: 3,1,2,4
+    #       so the 'winners' go to the gold final and the losers to the
+    #       bronze final.
+    spec = spec.decode('utf-8','replace')
+    ret = u''
     spec = reformat_placelist(spec)
     # pass 1: expand ranges
     for nr in spec.split():
-        if spec.find('-') >= 0:
+        if u'-' in spec:
             # try for a range...
             l = None
             n = None
-            for r in nr.split('-'):
+            for r in nr.split(u'-'):
                 if l is not None:
                     if l.isdigit() and r.isdigit():
                         start = int(l)
@@ -497,43 +351,49 @@ def placeset(spec=''):
                         if start < end:
                             c = start
                             while c < end:
-                                ret += ' ' + str(c)
+                                ret += u' ' + unicode(c)
                                 c += 1
                         else:
-                            ret += ' ' + l	# give up on last val
+                            ret += u' ' + l	# give up on last val
                     else:
                         # one or both not ints
-                        ret += ' ' + l
+                        ret += u' ' + l
                 else:
                     pass
                 l = r
             if l is not None: # catch final value
-                ret += ' ' + l
+                ret += u' ' + l
         else:
-            ret += ' ' + nr
+            ret += u' ' + nr
     # pass 2: filter out non-numbers
-    rset = set()
+    rset = []
     for i in ret.split():
         if i.isdigit():
-            rset.add(int(i))
+            ival = int(i)
+            if ival not in rset:
+                rset.append(ival)
     return rset
 
 def reformat_placelist(placestr):
     """Filter and return a canonically formatted place list."""
-    if placestr.find('-') < 0:		# This is the 'normal' case!
+    placestr = placestr.decode('utf-8','replace')
+    if u'-' not in placestr:		# This is the 'normal' case!
         return reformat_biblist(placestr)
     # otherwise, do the hard substitutions...
-    placestr = placestr.translate(PLACELIST_TRANS).strip()
+    placestr = placestr.translate(PLACELIST_UTRANS).strip()
     placestr = re.sub(r'\s*\-\s*', r'-', placestr)	# remove surrounds
     placestr = re.sub(r'\-+', r'-', placestr)		# combine dupes
-    return ' '.join(placestr.strip('-').split())
+    return u' '.join(placestr.strip(u'-').split())
 
 def confopt_bool(confstr):
     """Check and return a boolean option from config."""
-    if confstr.lower() in ['yes', 'true', '1']:
-        return True
+    if type(confstr) in [str, unicode]:
+        if confstr.lower() in ['yes', 'true', '1']:
+            return True
+        else:
+            return False
     else:
-        return False
+        return bool(confstr)
 
 def plural(count=0):
     """Return plural extension for provided count."""
@@ -547,7 +407,7 @@ def confopt_float(confstr, default=None):
     ret = default
     try:
         ret = float(confstr)
-    except ValueError:
+    except:	# catches the float(None) problem
         pass
     return ret
 
@@ -561,22 +421,18 @@ def confopt_distunits(confstr):
 def confopt_posint(confstr, default=None):
     """Check and return a valid positive integer."""
     ret = default
-    if confstr.isdigit():
+    try:
         ret = int(confstr)
+    except:
+        pass	# ignore errors and fall back on default
     return ret
 
 def confopt_dist(confstr, default=None):
     """Check and return a valid distance unit."""
-    ret = default
-    if confstr.isdigit():
-        ret = int(confstr)
-    return ret
+    return confopt_posint(confstr, default)
 
 def confopt_chan(confstr, default=None):
     """Check and return a valid timing channel id string."""
-    from metarace import timy 	# for channel info
-				# BAD BAD BAD: strops should have no
-				# external dependencies
     ret = timy.chan2id(default)
     ival = timy.chan2id(confstr)
     if ival != timy.CHAN_UNKNOWN:
@@ -599,27 +455,35 @@ def confopt_list(confstr, list=[], default=None):
             break
     return ret
 
-def bibstr2bibser(bibstr=''):
+def bibstr2bibser(bibstr=u''):
     """Split a bib.series string and return bib and series."""
-    a = bibstr.strip().split('.')
-    ret_bib = ''
-    ret_ser = ''
+    a = bibstr.strip().split(u'.')
+    ret_bib = u''
+    ret_ser = u''
     if len(a) > 0:
         ret_bib = a[0]
     if len(a) > 1:
         ret_ser = a[1]
     return (ret_bib, ret_ser)
 
-def bibser2bibstr(bib='', ser=''):
+def lapstring(lapcount=None):
+    lapstr = u''
+    if lapcount:
+        lapstr = unicode(lapcount) + u' Lap'
+        if lapcount > 1:
+            lapstr += u's'
+    return lapstr
+
+def bibser2bibstr(bib=u'', ser=u''):
     """Return a valid bib.series string."""
     ret = bib
-    if ser != '':
-        ret += '.' + ser
+    if ser != u'':
+        ret += u'.' + ser
     return ret
 
-def titlesplit(src='', linelen=22):
+def titlesplit(src=u'', linelen=24):
     """Split a string on word boundaries to try and fit into 3 fixed lines."""
-    ret = ['', '', '']
+    ret = [u'', u'', u'']
     words = src.split()
     wlen = len(words)
     if wlen > 0:
@@ -634,8 +498,7 @@ def titlesplit(src='', linelen=22):
                     break
                 ret[line] = word
             else:
-                ret[line] += ' ' + word
-
+                ret[line] += u' ' + word
     return ret
 
 class countback(object):
