@@ -118,20 +118,20 @@ QUANT_FW = [2, 4, 5, 6, 7, 8]
 QUANT_TWID = [8, 10, 11, 12, 13, 14]
 QUANT_PAD = ['     ', '   ', '  ', ' ', '', '']
 QUANT_OPAD = ['    ', '  ', ' ', '', '', '']
-TOD_RE=re.compile(r'^(?:(?:(\d{1,2})[h:-])?(\d{1,2})[:\'-])?(\d{1,2}(?:[\.\"]\d+)?)$')
+TOD_RE=re.compile(r'^(?:(?:(\d{1,2})[h:-])?(\d{1,2})[:\'-])?(\d{1,2}(?:[\.\"]\d+)?)$',flags=re.UNICODE)
 
 
-def str2tod(timeval=''):
+def str2tod(timeval=u''):
     """Return tod for given string without fail."""
     ret = None
-    if timeval is not None and timeval != '':
+    if timeval is not None and timeval != u'':
         try:
             ret = tod(timeval)
         except:
             pass
     return ret
 
-def dec2str(dectod=None, places=4, zeros=False, hoursep='h', minsep=':'):
+def dec2str(dectod=None, places=4, zeros=False, hoursep=u'h', minsep=u':'):
     """Return formatted string for given tod decimal value.
 
     Convert the decimal number dectod to a time string with the
@@ -153,25 +153,25 @@ def dec2str(dectod=None, places=4, zeros=False, hoursep='h', minsep=':'):
     
     if dectod is not None: 		# conditional here?
         if zeros or dectod >= 3600:	# NOTE: equal compares fine w/decimal
-            fmt = '{0}'+hoursep+'{1:02}'+minsep+'{2:0{3}}' # 'HHhMM:SS.dcmz'
+            fmt = u'{0}'+hoursep+u'{1:02}'+minsep+u'{2:0{3}}' # 'HHhMM:SS.dcmz'
             if zeros:
-                fmt = '{0:02}'+hoursep+'{1:02}'+minsep+'{2:0{3}}'# '00h00:0S.dcmz' (specialcase)
+                fmt = u'{0:02}'+hoursep+u'{1:02}'+minsep+u'{2:0{3}}'# '00h00:0S.dcmz' (specialcase)
             strtod = fmt.format(int(dectod)//3600,
                 (int(dectod)%3600)//60,
                 dectod.quantize(QUANT[places],
                 rounding=decimal.ROUND_FLOOR)%60,
                 QUANT_FW[places])
         elif dectod >= 60:	# MM:SS.dcmz
-            strtod = ('{0}'+minsep+'{1:0{2}}').format(int(dectod)//60,
+            strtod = (u'{0}'+minsep+u'{1:0{2}}').format(int(dectod)//60,
                 dectod.quantize(QUANT[places],
                 rounding=decimal.ROUND_FLOOR)%60,
                 QUANT_FW[places])
         else: 			# SS.dcmz or -SSSSS.dcmz
-            strtod = '{0}'.format(dectod.quantize(QUANT[places],
+            strtod = u'{0}'.format(dectod.quantize(QUANT[places],
                 rounding=decimal.ROUND_FLOOR))
     return strtod
 
-def str2dec(timestr=''):
+def str2dec(timestr=u''):
     """Return decimal for given string.
 
     Convert the time of day value represented by the string supplied
@@ -193,8 +193,8 @@ def str2dec(timestr=''):
 
     """
     dectod=None
-    timestr=timestr.strip()
-    if timestr == 'now':
+    timestr=timestr.strip()	# assumes string
+    if timestr == u'now':
         ltoft = time.localtime().tm_isdst * 3600	# DST Hack
         dectod = decimal.Decimal(str(
                     (time.time() - (time.timezone - ltoft)) % 86400))
@@ -202,7 +202,7 @@ def str2dec(timestr=''):
     else:
         m = TOD_RE.match(timestr)
         if m is not None:
-            dectod = decimal.Decimal(m.group(3).replace('"', '.'))
+            dectod = decimal.Decimal(m.group(3).replace(u'"', u'.'))
             dectod += decimal.Decimal(m.group(2) or 0) * 60
             dectod += decimal.Decimal(m.group(1) or 0) * 3600
         else:
@@ -213,7 +213,7 @@ def str2dec(timestr=''):
 
 class tod(object):
     """A class for representing time of day and RFID events."""
-    def __init__(self, timeval=0, index='', chan='', refid=''):
+    def __init__(self, timeval=0, index=u'', chan=u'', refid=u''):
         """Construct tod object.
 
         Keyword arguments:
@@ -224,10 +224,10 @@ class tod(object):
 
         """
 
-        self.index = str(index)[0:4]
-        self.chan = str(chan)[0:3]
+        self.index = index[0:4]	# numeric raises exception here
+        self.chan = chan[0:3]
         self.refid = refid
-        if type(timeval) is str:
+        if type(timeval) in [unicode, str]:
             self.timeval = str2dec(timeval)
         elif type(timeval) is tod:
             self.timeval = timeval.timeval
@@ -239,10 +239,14 @@ class tod(object):
         """Return a normalised tod string."""
         return self.refstr()
 
+    def __unicode__(self):
+        """Return a normalised tod string."""
+        return self.refstr()
+
     def __repr__(self):
         """Return object representation string."""
-        return "tod('{0}', '{1}', '{2}', '{3}')".format(str(self.timeval),
-            str(self.index), str(self.chan), str(self.refid))
+        return "tod({0}, {1}, {2}, {3})".format(repr(self.timeval),
+            repr(self.index), repr(self.chan), repr(self.refid))
 
     def refstr(self, places=4):
         """Return 'normalised' string form.
@@ -252,7 +256,7 @@ class tod(object):
         [0, 1, 2, 3, 4, 5]
 
         """
-        return '{0: >4} {1: <3} {2} {3}'.format(self.index, self.chan,
+        return u'{0: >4} {1: <3} {2} {3}'.format(self.index, self.chan,
                 self.timestr(places), self.refid)
 
     def truncate(self, places=4):
@@ -275,39 +279,39 @@ class tod(object):
         return (self.timeval / 60).quantize(QUANT[places],
                                             rounding=decimal.ROUND_FLOOR)
 
-    def timestr(self, places=4, zeros=False, hoursep='h', minsep=':'):
+    def timestr(self, places=4, zeros=False, hoursep=u'h', minsep=u':'):
         """Return time string component of the tod, whitespace padded."""
-        return '{0: >{1}}{2}'.format(dec2str(self.timeval, places, zeros,
+        return u'{0: >{1}}{2}'.format(dec2str(self.timeval, places, zeros,
                                              hoursep, minsep),
             QUANT_TWID[places], QUANT_PAD[places])
 
-    def omstr(self, places=3, zeros=False, hoursep=':', minsep=':'):
+    def omstr(self, places=3, zeros=False, hoursep=u':', minsep=u':'):
         """Return the 12 digit 'omega' style time string for gemini."""
         if places > 3:
             places = 3		# Hack to clamp to 12 dig
-        return '{0: >{1}}{2}'.format(dec2str(self.timeval, places, zeros,
+        return u'{0: >{1}}{2}'.format(dec2str(self.timeval, places, zeros,
                                              hoursep, minsep),
             QUANT_TWID[places], QUANT_OPAD[places])
 
     def meridian(self):
         """Return a 12hour time of day with meridian."""
-        med = 'am'
+        med = u'am'
         if self.timeval >= 43200:
-            med = 'pm'
+            med = u'pm'
         tv = self.timeval % 43200
         if tv < 3600:
             tv += 43200	# 12am/12pm 
-        return dec2str(tv, 0, hoursep=':', minsep=':') + med
+        return dec2str(tv, 0, hoursep=u':', minsep=u':') + med
 
-    def rawtime(self, places=4, zeros=False, hoursep='h', minsep=':'):
+    def rawtime(self, places=4, zeros=False, hoursep=u'h', minsep=u':'):
         """Return time string component of the tod, without padding."""
         return dec2str(self.timeval, places, zeros, hoursep, minsep)
 
     def speedstr(self, dist=200):
         """Return an average speed estimate for the provided distance."""
         if self.timeval == 0:
-            return '---.-- km/h'
-        return '{0:7.2f} km/h'.format(3.6 * float(dist) / float(self.timeval))
+            return '---.- km/h'
+        return '{0:5.1f} km/h'.format(3.6 * float(dist) / float(self.timeval))
 
     def copy(self):
         """Return a copy of the supplied tod."""
@@ -363,7 +367,7 @@ class tod(object):
                 oft = self.timeval - other.timeval
             else:
                 oft = 86400 - other.timeval + self.timeval
-            return tod(timeval=oft, index='', chan='NET', refid='')
+            return tod(timeval=oft, index=u'', chan=u'NET', refid=u'')
         else:
             raise TypeError('Cannot subtract {0} from tod.'.format(
                                 str(type(other).__name__)))
@@ -377,7 +381,7 @@ class tod(object):
         """
         if type(other) is tod:
             oft = (self.timeval + other.timeval) % 86400
-            return tod(timeval=oft, index='', chan='ToD', refid='')
+            return tod(timeval=oft, index=u'', chan=u'ToD', refid=u'')
         else:
             raise TypeError('Cannot add {0} to tod.'.format(
                                 str(type(other).__name__)))
@@ -389,11 +393,11 @@ class agg(tod):
        Promotion to aggregate time is 'sticky' and may be negative.
 
     """
-    def __init__(self, timeval=0, index='', chan='', refid=''):
-        self.index = str(index)[0:4]
-        self.chan = str(chan)[0:3]
+    def __init__(self, timeval=0, index=u'', chan=u'', refid=u''):
+        self.index = index[0:4]
+        self.chan = chan[0:3]
         self.refid = refid
-        if type(timeval) is str:
+        if type(timeval) in [unicode, str]:
             self.timeval = str2dec(timeval)
         elif type(timeval) is tod:
             self.timeval = timeval.timeval
@@ -403,53 +407,54 @@ class agg(tod):
     def truncate(self, places=4):
         """Return a new ToD object with a truncated time value."""
         return agg(timeval=self.timeval.quantize(QUANT[places],
-                rounding=decimal.ROUND_FLOOR), index='', chan='AGG', refid='')
+                rounding=decimal.ROUND_FLOOR), index=u'',
+                                 chan=u'AGG', refid=u'')
 
     def __add__(self, other):
         """Compute addition and return aggregate."""
         if type(other) is tod or type(other) is agg:
-            ret = agg(index='', chan='AGG', refid='')
+            ret = agg(index=u'', chan=u'AGG', refid=u'')
             ret.timeval = self.timeval + other.timeval
             return ret
         else:
-            raise TypeError('Cannot add {0} to agg.'.format(
+            raise TypeError(u'Cannot add {0} to agg.'.format(
                                 str(type(other).__name__)))
 
     def __sub__(self, other):
         """Compute subtraction and return aggregate."""
         if type(other) is tod or type(other) is agg:
-            ret = agg(index='', chan='AGG', refid='')
+            ret = agg(index=u'', chan=u'AGG', refid=u'')
             ret.timeval = self.timeval - other.timeval
             return ret
         else:
-            raise TypeError('Cannot subtract {0} from agg.'.format(
+            raise TypeError(u'Cannot subtract {0} from agg.'.format(
                                 str(type(other).__name__)))
 
 # ToD 'constants'
 ZERO = tod()
-MAX = tod('23h59:59.9999')
+MAX = tod(u'23h59:59.9999')
 
 # Fake times for special cases
 # these are impossible tods that still sort correctly
 FAKETIMES = {
- 'catch':ZERO,
- 'max':MAX.copy(),
- 'caught':MAX.copy(),
- 'rel':MAX.copy(),	# last place -> checka
- 'abort':MAX.copy(),	# similar to dnf?
- 'otl':MAX.copy(),	# outside time limit / hd
- 'dsq':MAX.copy(),	# disqualified
- 'dnf':MAX.copy(),	# did not finish
- 'dns':MAX.copy()}	# did not start
-extra = decimal.Decimal('0.00001')
-cof = decimal.Decimal('0.00001')
-for c in ['caught', 'rel', 'abort', 'otl', 'dsq', 'dnf', 'dns']:
+ u'catch':ZERO,
+ u'max':MAX.copy(),
+ u'caught':MAX.copy(),
+ u'rel':MAX.copy(),	# last place -> checka
+ u'abort':MAX.copy(),	# similar to dnf?
+ u'otl':MAX.copy(),	# outside time limit / hd
+ u'dsq':MAX.copy(),	# disqualified
+ u'dnf':MAX.copy(),	# did not finish
+ u'dns':MAX.copy()}	# did not start
+extra = decimal.Decimal(u'0.00001')
+cof = decimal.Decimal(u'0.00001')
+for c in [u'caught', u'rel', u'abort', u'otl', u'dsq', u'dnf', u'dns']:
     FAKETIMES[c].timeval += cof
     cof += extra
 
 class todlist():
     """ToD list helper class for managing splits and ranks."""
-    def __init__(self, lbl=''):
+    def __init__(self, lbl=u''):
         self.__label = lbl
         self.__store = []
 
@@ -462,7 +467,7 @@ class todlist():
     def __getitem__(self, key):
         return self.__store[key]
 
-    def rank(self, bib, series=''):
+    def rank(self, bib, series=u''):
         """Return current 0-based rank for given bib."""
         ret = None
         i = 0
@@ -480,15 +485,15 @@ class todlist():
     def clear(self):
         self.__store = []
 
-    def remove(self, bib, series=''):
+    def remove(self, bib, series=u''):
         i = 0
         while i < len(self.__store):
-            if self.__store[i].refid == bib and self.__store[i].index == series:
+            if self.__store[i].refid==bib and self.__store[i].index==series:
                 del self.__store[i]
             else:
                 i += 1
 
-    def insert(self, t, bib=None, series=''):
+    def insert(self, t, bib=None, series=u''):
         """Insert t into ordered list."""
         ret = None
         if t in FAKETIMES: # re-assign a coded 'finish'
@@ -517,7 +522,7 @@ if __name__ == "__main__":
                '3:45.6789',    '3-45.6789',
                '3:45',         '3-45',
                  '45.6789',        '5.6',
-                 '45',
+                 '45', FAKETIMES['dnf'], FAKETIMES[u'dnf'],
             1.4, float('1.4'), decimal.Decimal('1.4'), '1.4',
             10123, float('10123'), decimal.Decimal('10123'), '10123',
             10123.456, float('10123.456'),
@@ -552,4 +557,6 @@ if __name__ == "__main__":
     c = b.copy()
     print ('\t     c: '+ str(c))
     print ('\t   avg: '+ (b-a).speedstr())
+    print ('\tfake dns == make dns: ' + repr(FAKETIMES['dns'] == tod(FAKETIMES['dns'])))
+    print ('\tfake dns > fake dnf: ' + repr(FAKETIMES['dns'] > FAKETIMES['dnf']))
 
