@@ -135,6 +135,35 @@ class uscbsrv(threading.Thread):
         """Send a rider vector to the announcer."""
         self.sendmsg(unt4.unt4(header=header_txt,text=chr(unt4.US).join(rvec)))
 
+    def gfx_overlay(self, newov, chan=u'#tscb'):
+        """Update graphic channel overlay."""
+        self.queue.put_nowait(('MSG', 
+                   unt4.unt4(header=u'overlay', text=unicode(newov)).pack(),
+                       chan))
+
+    def gfx_clear(self, chan=u'#tscb'):
+        """Update graphic channel overlay."""
+        self.queue.put_nowait(('MSG', unt4.GENERAL_CLEARING, chan))
+
+    def gfx_set_title(self, title, chan=u'#tscb'):
+        """Update graphic channel title."""
+        self.queue.put_nowait(('MSG', 
+                    unt4.unt4(header=u'set_title', text=title).pack(),
+                       chan))
+
+    def gfx_add_row(self, rvec, chan=u'#tscb'):
+        """Update graphic channel title."""
+        ovec = []
+        for c in rvec:	# replace nulls and empties
+            nc = u''
+            if c:	# but assume strings?
+                nc = c
+            ovec.append(nc)
+        self.queue.put_nowait(('MSG', 
+                       unt4.unt4(header=u'add_row', 
+                                 text=chr(unt4.US).join(ovec)).pack(),
+                       chan))
+
     def setline(self, line, msg):
         """Set the specified DHI database line to msg."""
         msg = msg[0:self.linelen].ljust(self.linelen)
@@ -176,6 +205,10 @@ class uscbsrv(threading.Thread):
         """Suspend calling thread until cqueue is empty."""
         self.queue.join()
 
+    def add_channel(self, addchan=''):
+        """Request additional channel."""	# HACK -> Perth GP
+        self.queue.put_nowait(('ADDCHAN', addchan))
+
     def rejoin_channel(self, newchan=''):
         """Request another attempt to join channel."""
         self.queue.put_nowait(('CHAN', newchan))
@@ -206,6 +239,8 @@ class uscbsrv(threading.Thread):
             self.connect_pending = False # flags queue processing ok
             self.log.debug('Joined channel ' + str(e.target()))
             self.dumbcnt = 0
+        else:
+            self.log.debug('Joined additional channel ' + str(e.target()))
 
     def channel_part_cb(self, c, e):
         """Register channel part."""
@@ -231,7 +266,7 @@ class uscbsrv(threading.Thread):
                 if idx >= 0:
                     msgtxt = self.pub_rdbuf[0:idx+1]
                     self.pub_rdbuf = self.pub_rdbuf[idx+1:]
-                    glib.idle_add(self.pub_cb, unt4.unt4(unt4str=msgtxt))
+                    glib.idle_add(self.pub_cb, unt4.unt4(unt4str=msgtxt.decode('utf-8','replace')))
                     self._pacing(0.0)
         
     def privmsg_cb(self, c, e):
@@ -330,6 +365,9 @@ class uscbsrv(threading.Thread):
         """Return true if connected and in channel."""
         return self.ic.is_connected() and self.chanstatus
 
+    def _addchan(self, adchan):
+        self.ic.join(adchan)
+
     def _rechan(self):
         if self.localsrv:
             self.ic.oper('metairc', 'ogfPOHYkaw')
@@ -394,11 +432,17 @@ class uscbsrv(threading.Thread):
                     self.queue.task_done()
                     if m[0] == 'MSG' and self.host != '':
                         ## TODO : split message > 450 ?
-                        self.ic.privmsg(self.channel, unt4.encode(m[1]))
+                        chan = self.channel
+                        if len(m) > 2 and m[2]:
+                            chan = m[2]
+                        self.ic.privmsg(chan, unt4.encode(m[1]).encode('utf-8'))
                     elif m[0] == 'CHAN':
                         if m[1] != '':
                             self.channel = m[1]
                         self._rechan()
+                    elif m[0] == 'ADDCHAN':
+                        if m[1] != '':
+                            self._addchan(m[1])
                     elif m[0] == 'EXIT':
                         self.log.debug('Request to close : ' + str(m[1]))
                         self.running = False
